from __future__ import annotations

import mimetypes

from typing_extensions import Hashable, Sequence, TypeVar


class Content:
    """通用内容基类"""


ContentT = TypeVar("ContentT", bound=Content)


class TextContent(Content):
    """文本内容"""

    def __init__(self, text: str) -> None:
        """初始化文本内容

        :param text: 文本
        """
        super().__init__()
        self.text = text


class MediaContent(Content):
    """多媒体内容"""

    def __init__(
        self,
        *,
        name: str,
        url: str | None = None,
        raw: bytes | None = None,
        mimetype: str | None = None,
    ) -> None:
        """初始化多媒体内容

        :param name: 多媒体内容的名称
        :param raw: 多媒体内容的二进制内容
        :param url: 多媒体内容的网络地址（和 `raw` 参数二选一）
        :param mimetype: 多媒体内容的 mimetype，为空则根据 `name` 自动检测
        """
        super().__init__()
        self.name = name
        self.url = url
        self.val = raw if raw is not None else None
        if mimetype is None:
            self.mimetype, _ = mimetypes.guess_type(self.name)


class ImageContent(MediaContent):
    """图像内容，初始化方法参考基类"""


class AudioContent(MediaContent):
    """音频内容，初始化方法参考基类"""


class VoiceContent(AudioContent):
    """语音内容，初始化方法参考基类"""


class VideoContent(MediaContent):
    """视频内容，初始化方法参考基类"""


class FileContent(Content):
    """文件内容"""

    def __init__(self, name: str, flag: Hashable) -> None:
        """初始化文件内容

        :param name: 文件名
        :param flag: 文件的唯一标记
        """
        super().__init__()
        self.flag = flag
        self.name = name

    def __repr__(self) -> str:
        return f"[melobot File: {self.name} at {self.flag}]"


class ReferContent(Content):
    """引用内容"""

    def __init__(self, prompt: str, flag: Hashable, contents: Sequence[Content]) -> None:
        """初始化引用内容

        :param prompt: 引用的文本提示
        :param flag: 引用的唯一标记
        :param contents: 附加的通用内容序列
        """
        super().__init__()
        self.flag = flag
        self.prompt = prompt
        self.sub_contents = contents

    def __repr__(self) -> str:
        return f"[melobot Refer: {self.prompt} at {self.flag}]"


class ResourceContent(Content):
    """资源内容"""

    def __init__(self, name: str, url: str) -> None:
        """初始化资源内容

        :param name: 名称
        :param url: 网络地址
        """
        super().__init__()
        self.url = url
        self.name = name

    def __repr__(self) -> str:
        return f"[melobot Resource: {self.name} at {self.url}]"



from dataclasses import dataclass, field

from melobot.io import EchoPacket as RootEchoPak
from melobot.io import InPacket as RootInPak
from melobot.io import OutPacket as RootOutPak

from ..const import PROTOCOL_IDENTIFIER


@dataclass(kw_only=True)
class InPacket(RootInPak):  # type: ignore[override]
    data: dict
    protocol: str = PROTOCOL_IDENTIFIER


@dataclass(kw_only=True)
class OutPacket(RootOutPak):  # type: ignore[override]
    data: str
    action_type: str
    action_params: dict
    echo_id: str | None = None
    protocol: str = PROTOCOL_IDENTIFIER


@dataclass(kw_only=True)
class EchoPacket(RootEchoPak):  # type: ignore[override]
    action_type: str = ""
    data: dict = field(default_factory=dict)
    protocol: str = PROTOCOL_IDENTIFIER



import json
from asyncio import Queue, create_task

import websockets

from melobot.ctx import LoggerCtx
from melobot.log.base import Logger
from melobot.protocols.onebot.v11.adapter import action
from melobot.protocols.onebot.v11.io.forward import ForwardWebSocketIO
from melobot.protocols.onebot.v11.io.packet import OutPacket
from melobot.utils import singleton
from tests.base import *

_TEST_EVENT_DICT = {
    "time": 1725292489,
    "self_id": 123456,
    "post_type": "message",
    "message_type": "group",
    "sub_type": "normal",
    "sender": {
        "age": 0,
        "nickname": "这是一个群昵称",
        "sex": "unknown",
        "user_id": 1574260633,
        "area": "",
        "card": "",
        "level": "",
        "role": "member",
        "title": "",
    },
    "message_id": -1234567890,
    "font": 0,
    "message": "",
    "user_id": 1574260633,
    "anonymous": None,
    "group_id": 535705163,
    "raw_message": "",
}
_TEST_ACTION = action.SendMsgAction("hello world!", 123)
_TEST_ECHO_DICT = {
    "time": 1725292489,
    "status": "ok",
    "retcode": 0,
    "data": {"hello": "world!"},
}


_IN_BUF = Queue()
_OUT_BUF = Queue()


@singleton
class MockWebsocket:
    close_timeout = 0

    @staticmethod
    async def get(*args, **kwargs):
        return MockWebsocket()

    async def send(self, data: str) -> None:
        await _OUT_BUF.put(data)

    async def recv(self) -> str:
        return await _IN_BUF.get()

    async def close(self):
        return

    async def wait_closed(self) -> None:
        return


async def test_forward_ws(monkeypatch) -> None:
    with LoggerCtx().unfold(Logger()):
        monkeypatch.setattr(websockets, "connect", MockWebsocket.get)
        io = ForwardWebSocketIO("ws://example.com", max_retry=3, retry_delay=1)
        async with io:

            await _IN_BUF.put(json.dumps(_TEST_EVENT_DICT))
            pak = await io.input()
            assert pak.data == _TEST_EVENT_DICT

            pak = await io.output(
                OutPacket(
                    data=_TEST_ACTION.extract(),
                    action_type=_TEST_ACTION.extract()["action"],
                    action_params=_TEST_ACTION.extract()["params"],
                )
            )
            await _OUT_BUF.get() == _TEST_ACTION.flatten()
            assert pak.noecho

            _TEST_ACTION.set_echo(True)
            t = create_task(
                io.output(
                    OutPacket(
                        data=_TEST_ACTION.extract(),
                        action_type=_TEST_ACTION.extract()["action"],
                        action_params=_TEST_ACTION.extract()["params"],
                        echo_id=_TEST_ACTION.extract()["echo"],
                    )
                )
            )
            await _OUT_BUF.get() == _TEST_ACTION.flatten()
            await _IN_BUF.put(
                json.dumps(_TEST_ECHO_DICT | {"echo": _TEST_ACTION.extract()["echo"]})
            )
            pak = await t
            assert pak.data["data"]["hello"] == _TEST_ECHO_DICT["data"]["hello"]
            assert pak.ok == (_TEST_ECHO_DICT["status"] == "ok")
            assert pak.status == _TEST_ECHO_DICT["retcode"]



from melobot.protocols.onebot.v11.adapter import action
from melobot.protocols.onebot.v11.adapter import segment as seg
from tests.base import *


async def test_base():
    a = action.Action("test", {"hi": True, "you": 123.45})
    a.set_echo(True)
    assert a.need_echo
    assert a.flatten().startswith(
        '{"action": "test", "params": {"hi": true, "you": 123.45}'
    )


async def test_msgs_to_dicts():
    out = [{"type": "text", "data": {"text": "123"}}]

    msgs = "123"
    assert action.msgs_to_dicts(msgs) == out
    msgs = seg.Segment("text", text="123")
    assert action.msgs_to_dicts(msgs) == out
    msgs = [seg.Segment("text", text="123")]
    assert action.msgs_to_dicts(msgs) == out
    msgs = {"type": "text", "data": {"text": "123"}}
    assert action.msgs_to_dicts(msgs) == out
    msgs = [{"type": "text", "data": {"text": "123"}}]
    assert action.msgs_to_dicts(msgs) == out



from melobot.protocols.onebot.v11.adapter import echo
from melobot.protocols.onebot.v11.const import ACTION_TYPE_KEY_NAME
from tests.base import *


def head() -> dict:
    return {"status": "ok", "retcode": 0}


def ec(**kv_pairs):
    return head() | {"data": kv_pairs if len(kv_pairs) else None}


def fake_ec(**kv_pairs):
    return ec(**kv_pairs) | {"action_type": ""}


def li_ec(lis):
    return head() | {"data": lis}


def fake_li_ec(lis):
    return li_ec(lis) | {"action_type": ""}


async def test_empty():
    echo.EmptyEcho(**ec(), action_type="")


async def test_other():
    assert isinstance(
        echo.Echo.resolve({**ec(message_id=123), ACTION_TYPE_KEY_NAME: "send_msg"}),
        echo.SendMsgEcho,
    )
    assert isinstance(
        echo.Echo.resolve(
            {
                **ec(message_id=123, forward_id="abc"),
                ACTION_TYPE_KEY_NAME: "send_private_forward_msg",
            }
        ),
        echo.SendForwardMsgEcho,
    )

    assert echo.Echo.resolve(
        {
            **ec(
                time=123,
                message_type="private",
                message_id=123,
                real_id=456,
                sender={"user_id": 789, "nickname": "melody", "sex": "male", "age": 18},
                message="123&#91;45[CQ:node,user_id=10001000,nickname=某人,content=&#91;CQ:face&#44;id=123&#93;哈喽～]12345",
            ),
            ACTION_TYPE_KEY_NAME: "get_msg",
        }
    ).data["message"][0].data == {"text": "123[45"}

    assert not (
        echo.GetMsgEcho(
            **fake_ec(
                time=123,
                message_type="private",
                message_id=123,
                real_id=456,
                sender={
                    "user_id": 789,
                    "nickname": "melody",
                    "sex": "male",
                    "age": 18,
                },
                message=[
                    {"type": "text", "data": {"text": "12345"}},
                    {
                        "type": "node",
                        "data": {
                            "user_id": "10001000",
                            "nickname": "某人",
                            "content": [
                                {"type": "face", "data": {"id": "123"}},
                                {"type": "text", "data": {"text": "哈喽～"}},
                            ],
                        },
                    },
                ],
            ),
        )
        .data["sender"]
        .is_group_admin()
    )

    assert (
        echo.GetForwardMsgEcho(
            **fake_ec(
                message=[
                    {
                        "type": "node",
                        "data": {
                            "user_id": "10001000",
                            "nickname": "某人",
                            "content": [
                                {"type": "face", "data": {"id": "123"}},
                                {"type": "text", "data": {"text": "哈喽～"}},
                            ],
                        },
                    }
                ]
            )
        )
        .data["message"][0]
        .data["content"][0]
        .data["id"]
        == 123
    )

    echo.GetLoginInfoEcho(**fake_ec(user_id=123, nickname="melody"))
    echo.GetStrangerInfoEcho(
        **fake_ec(user_id=123, nickname="melody", sex="male", age=18)
    )
    echo.GetFriendListEcho(
        **fake_li_ec(
            [
                {"user_id": 123, "nickname": "melody", "remark": "123"},
                {"user_id": 456, "nickname": "jack", "remark": "123456"},
            ]
        )
    )
    echo.GetGroupInfoEcho(
        **fake_ec(group_id=123, group_name="test", member_count=12, max_member_count=100)
    )
    echo.GetGroupListEcho(
        **fake_li_ec(
            [
                {
                    "group_id": 123,
                    "group_name": "test",
                    "member_count": 12,
                    "max_member_count": 100,
                },
                {
                    "group_id": 123,
                    "group_name": "test",
                    "member_count": 12,
                    "max_member_count": 100,
                },
            ]
        )
    )

    base_info = {
        "group_id": 123,
        "user_id": 123,
        "nickname": "melody",
        "card": "123",
        "sex": "male",
        "age": 18,
        "area": "123",
        "join_time": 1234567,
        "last_sent_time": 1234567890,
        "level": "123",
        "role": "admin",
        "unfriendly": False,
        "title": "hello world",
        "title_expire_time": 1234567890,
        "card_changeable": True,
    }
    echo.GetGroupMemberInfoEcho(**fake_ec(**base_info))
    echo.GetGroupMemberListEcho(**fake_li_ec([base_info, base_info]))

    base_info = {
        "user_id": 123,
        "nickname": "melody",
        "avatar": "123",
        "description": "123",
    }
    echo.GetGroupHonorInfoEcho(
        **fake_ec(
            group_id=123,
            current_talkative={
                "user_id": 123,
                "nickname": "melody",
                "avatar": "123",
                "day_count": 123,
            },
            talkative_list=[base_info],
            performer_list=[base_info],
            legend_list=[base_info],
            strong_newbie_list=[base_info],
            emotion_list=[base_info],
        )
    )

    echo.GetCookiesEcho(**fake_ec(cookies="abc123"))
    echo.GetCsrfTokenEcho(**fake_ec(token=123))
    echo.GetCredentialsEcho(**fake_ec(csrf_token=123, cookies="abc123"))
    echo.GetRecordEcho(**fake_ec(file="123"))
    echo.GetImageEcho(**fake_ec(file="123"))
    echo.CanSendRecordEcho(**fake_ec(yes=True))
    echo.CanSendImageEcho(**fake_ec(yes=False))
    assert echo.GetStatusEcho(**fake_ec(online=True, good=True, nice=True)).data["nice"]
    assert (
        echo.GetVersionInfoEcho(
            **fake_ec(
                app_name="lgr",
                app_version="1.0.0",
                platform="linux",
                os_version="1.0.0",
                protocol_version="1.0.0",
            )
        ).data["os_version"]
        == "1.0.0"
    )



from asyncio import Queue

from melobot.protocols.onebot.v11.adapter import event
from melobot.protocols.onebot.v11.adapter.segment import AtSegment
from melobot.protocols.onebot.v11.utils import (
    AtMsgChecker,
    GroupRole,
    LevelRole,
    MsgCheckerFactory,
)
from tests.base import *

_CB_BUF = Queue()
_PRIVATE_EVENT_DICT = {
    "time": 1725292489,
    "self_id": 123456,
    "post_type": "message",
    "message_type": "private",
    "sub_type": "friend",
    "sender": {
        "age": 0,
        "sex": "unknown",
        "nickname": "这是一个昵称",
        "user_id": 1574260633,
    },
    "message_id": -1234567890,
    "font": 0,
    "message": "",
    "user_id": 1574260633,
    "raw_message": "",
}
_GRUOP_EVENT_DICT = {
    "time": 1725292489,
    "self_id": 123456,
    "post_type": "message",
    "message_type": "group",
    "sub_type": "normal",
    "sender": {
        "age": 0,
        "nickname": "这是一个群昵称",
        "sex": "unknown",
        "user_id": 1574260633,
        "area": "",
        "card": "",
        "level": "",
        "role": "member",
        "title": "",
    },
    "message_id": -1234567890,
    "font": 0,
    "message": "",
    "user_id": 1574260633,
    "anonymous": None,
    "group_id": 535705163,
    "raw_message": "",
}


@fixture
def msg_factory():
    return MsgCheckerFactory(
        owner=1,
        super_users=[2, 3],
        white_users=[4],
        black_users=[5],
        white_groups=[6, 7],
        fail_cb=lambda: _CB_BUF.put(True),
    )


def priv_e(uid: int):
    e = event.PrivateMessageEvent(**_PRIVATE_EVENT_DICT)
    e.sender.user_id = e.user_id = uid
    return e


def group_e(uid: int, gid: int, role: str = "member"):
    e = event.GroupMessageEvent(**_GRUOP_EVENT_DICT)
    e.group_id = gid
    e.sender.user_id = e.user_id = uid
    e.sender.role = role
    return e


async def test_msg_checker(msg_factory: MsgCheckerFactory):
    c1 = msg_factory.get_base(LevelRole.NORMAL)
    assert await c1.check(priv_e(1))
    assert await c1.check(priv_e(3))
    assert await c1.check(priv_e(4))
    assert not await c1.check(priv_e(5))
    _CB_BUF.get_nowait()
    assert await c1.check(priv_e(10))
    assert await c1.check(group_e(2, 6))

    c2 = msg_factory.get_base(LevelRole.WHITE)
    assert not await c2.check(priv_e(10))
    _CB_BUF.get_nowait()

    c3 = msg_factory.get_private(LevelRole.NORMAL)
    assert await c3.check(priv_e(1))
    assert not await c3.check(group_e(1, 6))
    _CB_BUF.get_nowait()

    c4 = msg_factory.get_group(LevelRole.NORMAL)
    assert not await c4.check(priv_e(1))
    _CB_BUF.get_nowait()
    assert await c4.check(group_e(1, 6))
    assert not await c4.check(group_e(1, 8))
    _CB_BUF.get_nowait()

    c5 = msg_factory.get_base(LevelRole.NORMAL, fail_cb=lambda: _CB_BUF.put(True))
    assert not await c5.check(priv_e(5))
    _CB_BUF.get_nowait()

    c6 = msg_factory.get_base(GroupRole.OWNER)
    assert not await c6.check(priv_e(1))
    _CB_BUF.get_nowait()
    assert await c6.check(group_e(1, 6, "owner"))
    assert not await c6.check(group_e(1, 6, "admin"))
    _CB_BUF.get_nowait()
    assert not await c6.check(group_e(1, 6, "member"))
    _CB_BUF.get_nowait()

    c7 = msg_factory.get_base(GroupRole.ADMIN)
    c8 = msg_factory.get_base(LevelRole.WHITE)
    assert await (c7 & c8).check(group_e(4, 6, "admin"))


def at_e(atid: int | str):
    e = priv_e(1)
    e.raw_message = f"[CQ:at,qq={atid}]"
    e.message.append(AtSegment(atid))
    return e


async def test_at_checker():
    c1 = AtMsgChecker(1)
    assert await c1.check(at_e(1))
    assert not await c1.check(at_e(2))

    c2 = AtMsgChecker("all")
    assert not await c2.check(at_e(1))
    assert await c2.check(at_e("all"))

    c3 = AtMsgChecker()
    assert await c3.check(at_e(1))
    assert await c3.check(at_e(2))



from __future__ import annotations

from asyncio import create_task, get_running_loop
from dataclasses import dataclass
from itertools import tee

from typing_extensions import Iterable, NoReturn, Sequence

from ..adapter.base import Event
from ..ctx import BotCtx, EventCompletion, FlowCtx, FlowRecord, FlowRecords
from ..ctx import FlowRecordStage as RecordStage
from ..ctx import FlowStatus, FlowStore
from ..di import DependNotMatched, inject_deps
from ..exceptions import FlowError
from ..log import LogLevel
from ..mixin import LogMixin
from ..typ.base import AsyncCallable, SyncOrAsyncCallable
from ..utils.base import to_async
from ..utils.common import get_obj_name

_FLOW_CTX = FlowCtx()


def node(func: SyncOrAsyncCallable[..., bool | None]) -> FlowNode:
    """处理结点装饰器，将当前异步可调用对象装饰为一个处理结点"""
    return FlowNode(func)


def no_deps_node(func: SyncOrAsyncCallable[..., bool | None]) -> FlowNode:
    """与 :func:`node` 类似，但是不自动为结点标记依赖注入。

    需要后续使用 :func:`.inject_deps` 手动标记依赖注入，
    这适用于某些对处理结点进行再装饰的情况
    """
    return FlowNode(func, no_deps=True)


class FlowNode:
    """处理流结点"""

    def __init__(
        self, func: SyncOrAsyncCallable[..., bool | None], no_deps: bool = False
    ) -> None:
        self.name = get_obj_name(func, otype="callable")
        self.processor: AsyncCallable[..., bool | None] = (
            to_async(func) if no_deps else inject_deps(func)
        )

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(name={self.name})"

    async def process(self, flow: Flow, completion: EventCompletion) -> None:
        event = completion.event
        try:
            status = _FLOW_CTX.get()
            records, store = status.records, status.store
        except _FLOW_CTX.lookup_exc_cls:
            records, store = FlowRecords(), FlowStore()

        with _FLOW_CTX.unfold(FlowStatus(flow, self, True, completion, records, store)):
            try:
                records.append(
                    FlowRecord(RecordStage.NODE_START, flow.name, self.name, event)
                )

                try:
                    ret = await self.processor()
                    records.append(
                        FlowRecord(RecordStage.NODE_FINISH, flow.name, self.name, event)
                    )
                except DependNotMatched as e:
                    ret = False
                    records.append(
                        FlowRecord(
                            RecordStage.DEPENDS_NOT_MATCH,
                            flow.name,
                            self.name,
                            event,
                            f"Real({e.real_type}) <=> Annotation({e.hint})",
                        )
                    )

                if ret in (None, True) and _FLOW_CTX.get().next_valid:
                    await nextn()

            except FlowContinued:
                await nextn()


@dataclass
class NodeInfo:
    nexts: list[FlowNode]
    in_deg: int
    out_deg: int

    def copy(self) -> NodeInfo:
        return NodeInfo(self.nexts, self.in_deg, self.out_deg)


class Flow(LogMixin):
    """处理流

    :ivar str name: 处理流的标识
    """

    def __init__(
        self,
        name: str,
        *edge_maps: Iterable[Iterable[FlowNode] | FlowNode],
        priority: int = 0,
    ) -> None:
        """初始化处理流

        :param name: 处理流的标识
        :param edge_maps: 边映射，遵循 melobot 的 graph edges 表示方法
        :param priority: 处理流的优先级
        """
        self.name = name
        self.graph: dict[FlowNode, NodeInfo] = {}
        self.priority = priority

        self._active = True

        _edge_maps = tuple(
            tuple((elem,) if isinstance(elem, FlowNode) else elem for elem in emap)
            for emap in edge_maps
        )
        edges = self._get_edges(_edge_maps)

        for n1, n2 in edges:
            self._add(n1, n2)

        if not self._valid_check():
            raise FlowError(f"定义的处理流 {self.name} 中存在环路")

    def __repr__(self) -> str:
        output = (
            f"{self.__class__.__name__}(name={self.name}, active={self._active}"
            f", pri={self.priority}, nums={len(self.graph)}"
        )

        if len(self.graph):
            output += f", starts=[{', '.join(n.name for n in self.starts)}])"
        else:
            output += ")"
        return output

    @property
    def starts(self) -> tuple[FlowNode, ...]:
        return tuple(n for n, info in self.graph.items() if info.in_deg == 0)

    @property
    def ends(self) -> tuple[FlowNode, ...]:
        return tuple(n for n, info in self.graph.items() if info.out_deg == 0)

    def update_priority(self, priority: int) -> None:
        """更新处理流优先级

        :param priority: 新优先级
        """
        BotCtx().get()._dispatcher.update(priority, self)

    def dismiss(self) -> None:
        """停用处理流

        停用后将无法处理任何新事件，也无法再次恢复使用
        """
        self._active = False

    def is_active(self) -> bool:
        """判断处理流是否处于可用状态

        :return: 是否可用
        """
        return self._active

    def link(self, flow: Flow, priority: int | None = None) -> Flow:
        """连接另一处理流返回新处理流，并设置新优先级

        :param flow: 连接的新流
        :param priority: 新优先级，若为空，则使用两者中较小的优先级
        :return: 新的处理流
        """
        _froms = self.ends
        _tos = flow.starts
        new_edges = tuple((n1, n2) for n1 in _froms for n2 in _tos)

        new_flow = Flow(
            f"{self.name} ~ {flow.name}",
            *new_edges,
            priority=priority if priority else min(self.priority, flow.priority),
        )

        for n1, info in (self.graph | flow.graph).items():
            if not len(info.nexts):
                new_flow._add(n1, None)
                continue
            for n2 in info.nexts:
                new_flow._add(n1, n2)

        if not self._valid_check():
            raise FlowError(f"定义的处理流 {self.name} 中存在环路")

        return new_flow

    def _get_edges(
        self, edge_maps: Sequence[Sequence[Iterable[FlowNode]]]
    ) -> list[tuple[FlowNode, FlowNode]]:
        edges: list[tuple[FlowNode, FlowNode]] = []

        for emap in edge_maps:
            iter1, iter2 = tee(emap, 2)
            try:
                next(iter2)
            except StopIteration:
                continue

            if len(emap) == 1:
                for n in emap[0]:
                    self._add(n, None)
                continue

            for from_seq, to_seq in zip(iter1, iter2):
                for n1 in from_seq:
                    for n2 in to_seq:
                        if (n1, n2) not in edges:
                            edges.append((n1, n2))

        return edges

    def _add(self, _from: FlowNode, to: FlowNode | None) -> None:
        from_info = self.graph.setdefault(_from, NodeInfo([], 0, 0))

        if to is not None:
            to_info = self.graph.setdefault(to, NodeInfo([], 0, 0))
            to_info.in_deg += 1
            from_info.out_deg += 1
            from_info.nexts.append(to)

    def _valid_check(self) -> bool:
        graph = {n: info.copy() for n, info in self.graph.items()}

        while len(graph):
            for n, info in graph.items():
                nexts, in_deg = info.nexts, info.in_deg

                if in_deg == 0:
                    graph.pop(n)
                    for next_n in nexts:
                        graph[next_n].in_deg -= 1
                    break

            else:
                return False

        return True

    async def _handle(self, event: Event) -> None:
        fut = get_running_loop().create_future()
        create_task(self._run(EventCompletion(event, fut, self)))
        await fut

    async def _run(self, completion: EventCompletion) -> None:
        if not len(self.starts):
            if (
                completion.owner_flow is self
                and not completion.under_session
                and not completion.completed.done()
            ):
                completion.completed.set_result(None)
            return

        event = completion.event
        try:
            status = _FLOW_CTX.get()
            records, store = status.records, status.store
        except _FLOW_CTX.lookup_exc_cls:
            records, store = FlowRecords(), FlowStore()

        with _FLOW_CTX.unfold(
            FlowStatus(self, self.starts[0], True, completion, records, store)
        ):
            try:
                records.append(
                    FlowRecord(
                        RecordStage.FLOW_START, self.name, self.starts[0].name, event
                    )
                )

                idx = 0
                while idx < len(self.starts):
                    try:
                        await self.starts[idx].process(self, completion)
                        idx += 1
                    except FlowRewound:
                        pass

                records.append(
                    FlowRecord(
                        RecordStage.FLOW_FINISH, self.name, self.starts[0].name, event
                    )
                )

            except FlowBroke:
                pass

            except Exception:
                self.logger.exception(f"事件处理流 {self.name} 发生异常")
                self.logger.generic_obj(
                    f"异常点 event {event.id}", event, level=LogLevel.ERROR
                )
                self.logger.generic_obj(
                    "异常点局部变量：",
                    {"completion": completion.__dict__, "cur_flow": self},
                    level=LogLevel.ERROR,
                )

            finally:
                if (
                    completion.owner_flow is self
                    and not completion.under_session
                    and not completion.completed.done()
                ):
                    completion.completed.set_result(None)


class _FlowSignal(BaseException): ...


class FlowBroke(_FlowSignal): ...


class FlowContinued(_FlowSignal): ...


class FlowRewound(_FlowSignal): ...


async def nextn() -> None:
    """运行下一处理结点（在处理流中使用）"""
    try:
        status = _FLOW_CTX.get()
        nexts = status.flow.graph[status.node].nexts
        if not status.next_valid:
            return

        idx = 0
        while idx < len(nexts):
            try:
                await nexts[idx].process(status.flow, status.completion)
                idx += 1
            except FlowRewound:
                pass

    except _FLOW_CTX.lookup_exc_cls:
        raise FlowError("此时不在活动的事件处理流中，无法调用下一处理结点") from None
    finally:
        status.next_valid = False


async def block() -> None:
    """阻止当前事件向更低优先级的处理流传播（在处理流中使用）"""
    status = _FLOW_CTX.get()
    status.records.append(
        FlowRecord(
            RecordStage.BLOCK, status.flow.name, status.node.name, status.completion.event
        )
    )
    status.completion.event.spread = False


async def stop() -> NoReturn:
    """立即停止当前处理流（在处理流中使用）"""
    status = _FLOW_CTX.get()
    status.records.append(
        FlowRecord(
            RecordStage.STOP, status.flow.name, status.node.name, status.completion.event
        )
    )
    status.records.append(
        FlowRecord(
            RecordStage.NODE_EARLY_FINISH,
            status.flow.name,
            status.node.name,
            status.completion.event,
        )
    )
    status.records.append(
        FlowRecord(
            RecordStage.FLOW_EARLY_FINISH,
            status.flow.name,
            status.node.name,
            status.completion.event,
        )
    )
    raise FlowBroke("事件处理流被安全地提早结束，请无视这个内部工作信号")


async def bypass() -> NoReturn:
    """立即跳过当前处理结点剩下的步骤，运行下一处理结点（在处理流中使用）"""
    status = _FLOW_CTX.get()
    status.records.append(
        FlowRecord(
            RecordStage.BYPASS,
            status.flow.name,
            status.node.name,
            status.completion.event,
        )
    )
    status.records.append(
        FlowRecord(
            RecordStage.NODE_EARLY_FINISH,
            status.flow.name,
            status.node.name,
            status.completion.event,
        )
    )
    raise FlowContinued("事件处理流安全地跳过结点执行，请无视这个内部工作信号")


async def rewind() -> NoReturn:
    """立即重新运行当前处理结点（在处理流中使用）"""
    status = _FLOW_CTX.get()
    status.records.append(
        FlowRecord(
            RecordStage.REWIND,
            status.flow.name,
            status.node.name,
            status.completion.event,
        )
    )
    status.records.append(
        FlowRecord(
            RecordStage.NODE_EARLY_FINISH,
            status.flow.name,
            status.node.name,
            status.completion.event,
        )
    )
    raise FlowRewound("事件处理流安全地重复执行处理结点，请无视这个内部工作信号")


async def flow_to(flow: Flow) -> None:
    """立即进入一个其他处理流（在处理流中使用）"""
    status = _FLOW_CTX.get()
    await flow._run(status.completion)



import importlib.resources
import re
import sys
from contextlib import ExitStack
from inspect import getabsfile
from os import PathLike, listdir, remove
from pathlib import Path
from time import time
from types import ModuleType

from typing_extensions import Any, Callable, Iterable, cast

from .._imp import Importer
from ..ctx import BotCtx, LoggerCtx
from ..exceptions import PluginAutoGenError, PluginLoadError
from ..utils.common import singleton
from .base import Plugin, PluginPlanner
from .ipc import AsyncShare, SyncShare


def plugin_get_attr(p_name: str, name: str) -> Any:
    obj = BotCtx().get().ipc_manager.get(p_name, name)
    if obj.static:
        return obj.get()
    return obj


_AUTOGEN_COMMENT = r"""# This file is @generated by melobot cli.
# It is not intended for manual editing.
"""


class PluginInitHelper:
    _BASE_INIT_PY_STR = (
        importlib.resources.files(__package__)
        .joinpath("__init__.template.py")
        .read_text(encoding="utf-8")
    )

    @staticmethod
    def _get_init_py_str() -> str:
        return re.sub(
            r"_VAR(\d+)",
            lambda matched: f"_{int(time()):#x}{matched.group(1)}",
            PluginInitHelper._BASE_INIT_PY_STR,
        )

    @staticmethod
    def _get_init_pyi_str(
        plugin_dir: Path,
        shares: Iterable[AsyncShare | SyncShare],
        funcs: Iterable[Callable],
    ) -> str:
        refs: dict[str, list[str]] = {}
        varnames: list[str] = []
        autoget_varnames: list[str] = []

        for share in shares:
            if share.static:
                autoget_varnames.append(share.name)

            share_located = Path(share.__obj_file__)
            try:
                parts1 = share_located.resolve(strict=True).parts
            except Exception as e:
                raise PluginAutoGenError(
                    f"无法定位共享对象所在的模块位置，对应插件：{plugin_dir}"
                ) from e

            parts2 = plugin_dir.parts
            imp_parts = list(parts1[len(parts2) :])
            imp_parts[-1] = imp_parts[-1].rstrip(".py")

            mod = Importer.import_mod(share.__obj_module__, share_located.parent)
            for k in dir(mod):
                v = getattr(mod, k)
                if v is share:
                    share_varname = k
                    break
            else:
                raise PluginAutoGenError(
                    f"共享对象 {share} 在所属模块的全局作用域不存在引用，生成 .pyi 失败，"
                    f"对应插件：{plugin_dir}"
                )

            li = refs.setdefault(f".{'.'.join(imp_parts)}", [])
            if share.static:
                li.append(f"{share_varname} as _{share.name}")
            elif share_varname != share.name:
                li.append(f"{share_varname} as {share.name}")
            else:
                li.append(share_varname)
            varnames.append(share.name)

        for func in funcs:
            func_located = Path(getabsfile(func))
            try:
                parts1 = func_located.resolve(strict=True).parts
            except Exception as e:
                raise PluginAutoGenError(
                    f"无法定位导出函数所在模块的位置，对应插件：{plugin_dir}"
                ) from e

            parts2 = plugin_dir.parts
            imp_parts = list(parts1[len(parts2) :])
            imp_parts[-1] = imp_parts[-1].rstrip(".py")
            if func.__name__.startswith("_"):
                raise PluginAutoGenError(
                    f"导出函数 {func} 的名称不能以 _ 开头，对应插件：{plugin_dir}"
                )

            mod = Importer.import_mod(func.__module__, func_located.parent)
            for k in dir(mod):
                v = getattr(mod, k)
                if v is func:
                    break
            else:
                raise PluginAutoGenError(
                    f"导出函数 {func} 在所属模块的全局作用域不存在引用，生成 .pyi 失败，"
                    f"对应插件：{plugin_dir}"
                )
            refs.setdefault(f".{'.'.join(imp_parts)}", []).append(func.__name__)
            varnames.append(func.__name__)

        imp_lines = "\n".join(
            f"from {namespace} import {name}"
            for namespace, obj_names in refs.items()
            for name in obj_names
        )
        gets_str = "\n".join(
            f"{varname} = _{varname}.get()" for varname in autoget_varnames
        )
        vars_str = f"__all__ = {repr(tuple(varnames))}"

        if not len(varnames):
            return ""

        return "\n\n".join((imp_lines, gets_str, vars_str)) + "\n"

    @staticmethod
    def run_init(*plugin_dirs: str | PathLike[str], load_depth: int = 1) -> None:
        # pylint: disable=cyclic-import
        from melobot.bot import Bot

        tmp_bot = Bot(enable_log=False)

        for p_dir in plugin_dirs:
            p_dir = Path(p_dir)
            if not p_dir.is_absolute():
                p_dir = Path.cwd().joinpath(p_dir)

            p_name = p_dir.parts[-1]
            p_conflicts = set(fname.split(".")[0] for fname in listdir(p_dir))
            pinit_path = p_dir.joinpath("__init__.py")
            pinit_typ_path = p_dir.joinpath("__init__.pyi")

            if p_name in sys.modules or p_name in sys.stdlib_module_names:
                raise PluginAutoGenError(
                    f"插件 {p_name} 与 python 内置模块或已加载模块重名，"
                    f"请修改名称（修改插件目录名）。对应插件：{p_dir}"
                )
            if not p_dir.joinpath("__plugin__.py").exists():
                raise PluginAutoGenError(
                    f"插件目录下不存在 __plugin__.py，无法解析。对应插件：{p_dir}"
                )
            if pinit_path.exists():
                remove(pinit_path)
            if pinit_typ_path.exists():
                remove(pinit_typ_path)

            with ExitStack() as ctx_stack:
                ctx_stack.enter_context(BotCtx().unfold(tmp_bot))
                ctx_stack.enter_context(LoggerCtx().unfold(tmp_bot.logger))

                prefix = ".".join(p_dir.parts[-load_depth:])
                p_entry_mod_name = f"{prefix}.__plugin__"
                p_entry_mod = Importer.import_mod(p_entry_mod_name, p_dir)
                for k in dir(p_entry_mod):
                    val = getattr(p_entry_mod, k)
                    if isinstance(val, PluginPlanner):
                        p_planner = val
                        p_shares = p_planner.shares
                        p_funcs = p_planner.funcs
                        break
                else:
                    raise PluginAutoGenError(
                        f"插件的 __plugin__.py 未实例化 {PluginPlanner.__name__} 类，"
                        f"无法解析。对应插件：{p_dir}"
                    )

                for share in p_shares:
                    if share.name in p_conflicts:
                        raise PluginAutoGenError(
                            f"插件的共享对象名 {share.name} 与插件根目录下的文件/目录名重复，"
                            "这将导致导入混淆，请修改共享对象名。"
                            f"对应插件：{p_dir}"
                        )
                for func in p_funcs:
                    if func.__name__ in p_conflicts:
                        raise PluginAutoGenError(
                            f"插件的导出函数名 {func.__name__} 与插件根目录下的文件/目录名重复，"
                            "这将导致导入混淆，请修改导出函数名"
                            f"对应插件：{p_dir}"
                        )

            try:
                pyi_content = PluginInitHelper._get_init_pyi_str(p_dir, p_shares, p_funcs)
                if pyi_content != "":
                    with open(pinit_typ_path, "w", encoding="utf-8") as fp:
                        fp.write(_AUTOGEN_COMMENT + pyi_content)
                    with open(pinit_path, "w", encoding="utf-8") as fp:
                        fp.write(_AUTOGEN_COMMENT + PluginInitHelper._get_init_py_str())

            except Exception:
                if pinit_path.exists():
                    remove(pinit_path)
                if pinit_typ_path.exists():
                    remove(pinit_typ_path)
                raise

            Importer.clear_cache()


P_PLANNER_ATTR = "__plugin_planner__"


@singleton
class PluginLoader:
    def __init__(self) -> None:
        self._dir_caches: dict[str, Path] = {}

    def _build_plugin(
        self, p_name: str, entry: ModuleType | None, p_planner: PluginPlanner | None
    ) -> tuple[Plugin, bool]:
        if p_planner is None:

            if not hasattr(entry, P_PLANNER_ATTR):
                for k in dir(entry):
                    val = getattr(entry, k)
                    if isinstance(val, PluginPlanner):
                        setattr(entry, P_PLANNER_ATTR, val)
                        break

                else:
                    entry = cast(ModuleType, entry)
                    if entry.__file__ is None:
                        p_dir = Path(entry.__path__[0])
                    else:
                        p_dir = Path(entry.__file__).parent

                    raise PluginLoadError(
                        f"插件的 __plugin__.py 未实例化 {PluginPlanner.__name__} 类，无法加载。"
                        f"对应插件：{p_dir}"
                    )

            p_planner = getattr(entry, P_PLANNER_ATTR)

        p_planner = cast(PluginPlanner, p_planner)
        if p_planner._built:
            p = p_planner._plugin
            LoggerCtx().get().debug(f"插件 {p.name} 已加载，重复加载将被跳过")
            return p, True

        p = p_planner.__p_build__(name=p_name)
        return p, False

    def load(
        self, plugin: ModuleType | str | PathLike[str] | PluginPlanner, load_depth: int
    ) -> tuple[Plugin, bool]:
        logger = LoggerCtx().get()
        _plugin_repr = repr(plugin)

        if isinstance(plugin, PluginPlanner):
            p_name = f"_DynamicPlugin_0x{id(plugin):0x}"
            return self._build_plugin(p_name, entry=None, p_planner=plugin)

        if isinstance(plugin, str):
            try:
                tmp_mod = Importer.import_mod(plugin)
                assert isinstance(tmp_mod, ModuleType)
                plugin = tmp_mod
            except BaseException:
                pass

        if isinstance(plugin, ModuleType):
            if plugin.__file__ is None:
                p_dir = Path(plugin.__path__[0])
            else:
                p_dir = Path(plugin.__file__).parent
        else:
            p_dir = Path(plugin)
            if not p_dir.is_absolute():
                p_dir = Path.cwd().joinpath(p_dir).resolve()

        p_name = p_dir.parts[-1]
        logger.debug(f"尝试加载来自 {repr(p_dir)} 的插件：{p_name}")

        if p_name in sys.stdlib_module_names:
            raise PluginLoadError(
                f"尝试加载的插件 {p_name} 与 Python 内置模块重名，"
                f"请修改名称（修改插件目录名）。对应插件：{p_dir}"
            )
        if not p_dir.joinpath("__plugin__.py").exists():
            raise PluginLoadError(
                f"插件目录下不存在 __plugin__.py，无法加载。对应插件：{p_dir}"
            )

        p_mod = Importer.get_cache(p_dir)
        if p_mod is not None:
            p_entry_mod_name = f"{p_mod.__name__}.__plugin__"
        else:
            prefix = ".".join(p_dir.parts[-load_depth:])
            p_entry_mod_name = f"{prefix}.__plugin__"

        if p_name in self._dir_caches and p_dir.resolve() != self._dir_caches[p_name]:
            raise PluginLoadError(
                f"试图加载一个与已加载插件同名的插件，名称： {p_name}，"
                f"加载参数：{_plugin_repr}，尝试加载的插件：{p_dir}"
            )

        p_entry_mod = Importer.import_mod(p_entry_mod_name, p_dir)
        p, is_repeat = self._build_plugin(p_name, entry=p_entry_mod, p_planner=None)
        if not is_repeat:
            self._dir_caches[p.name] = p_dir.resolve()
        return p, is_repeat



from ..ctx import LoggerCtx as _LoggerCtx
from .base import GenericLogger, Logger, LogLevel
from .patch import LazyLogMethod, LoguruPatch, StandardPatch, StructlogPatch, logger_patch


def get_logger() -> GenericLogger:
    """获取当前上下文中日志器

    :return: 日志器
    """
    return _LoggerCtx().get()



import inspect
from asyncio import Future, get_running_loop

from typing_extensions import Any, Callable, Generic, Self, cast

from ._hook import HookBus, HookEnumT
from .ctx import LoggerCtx
from .log.base import GenericLogger
from .typ.base import AsyncCallable, P, SyncOrAsyncCallable
from .utils.base import to_async


class LogMixin:
    @property
    def logger(self) -> GenericLogger:
        return LoggerCtx().get()


class FlagMixin:
    def __init__(self) -> None:
        self.__flag_mixin_flags__: dict[Any, dict[Any, Any]] = {}
        self.__flag_mixin_waitings__: dict[
            tuple[Any, Any], list[tuple[Any, Future, bool, bool]]
        ] = {}

    def __flag_waitings_fulfill__(self, namespace: Any, flag: Any, val: Any) -> None:
        waitings = self.__flag_mixin_waitings__.get((namespace, flag))
        if waitings is None:
            return

        for waiting in waitings:
            expect_val, signal, use_id, wait_val = waiting
            if not wait_val:
                signal.set_result(None)
                continue

            if use_id and val is expect_val:
                signal.set_result(None)
                continue

            if not use_id and val == expect_val:
                signal.set_result(None)
                continue

    def flag_set(
        self,
        namespace: Any,
        flag: Any,
        val: Any = None,
        strict: bool = True,
    ) -> None:
        """设置标记

        注：不同的对象并不共享 `namespace`，`namespace` 只适用于单个对象

        :param namespace: 命名空间
        :param flag: 标记
        :param val: 标记值
        :param strict: 严格模式，启用严格模式，则不允许 `flag` 标记已经存在
        """
        self.__flag_mixin_flags__.setdefault(namespace, {})

        if strict and flag in self.__flag_mixin_flags__[namespace].keys():
            raise ValueError(
                f"标记失败。对象 {self} 的命名空间 {namespace} 中已存在名为 {flag} 的标记"
            )

        self.__flag_mixin_flags__[namespace][flag] = val
        self.__flag_waitings_fulfill__(namespace, flag, val)

    def flag_set_default(self, namespace: Any, flag: Any, default: Any) -> None:
        """设置标记，并在标记不存在时使用 `default` 初始化

        注：不同的对象并不共享 `namespace`，`namespace` 只适用于单个对象

        :param namespace: 命名空间
        :param flag: 标记
        :param default: 标记不存在时的默认值
        """
        self.__flag_mixin_flags__.setdefault(namespace, {})
        self.__flag_mixin_flags__[namespace].setdefault(flag, default)
        val = self.__flag_mixin_flags__[namespace][flag]
        self.__flag_waitings_fulfill__(namespace, flag, val)

    def flag_get(
        self, namespace: Any, flag: Any, raise_exc: bool = True, default: Any = None
    ) -> Any:
        """获取标记值

        注：不同的对象并不共享 `namespace`，`namespace` 只适用于单个对象

        :param namespace: 命名空间
        :param flag: 标记
        :param raise_exc: 为 `True`，则在标记不存在时引发 `KeyError`
        :param default: 标记不存在时的默认值，只在 `raise_exc` 为 `False` 时有效
        :return: 标记值
        """
        try:
            return self.__flag_mixin_flags__[namespace][flag]
        except KeyError:
            if raise_exc:
                raise KeyError(
                    f"对象 {self} 的命名空间 {namespace} 中不存在名为 {flag} 的标记"
                ) from None
            return default

    def flag_check(
        self,
        namespace: Any,
        flag: Any,
        val: Any = None,
        check_val: bool = True,
        use_id: bool = False,
    ) -> bool:
        """检查标记

        注：不同的对象并不共享 `namespace`，`namespace` 只适用于单个对象

        :param namespace: 命名空间
        :param flag: 标记
        :param val: 标记值
        :param check_val: 为 `True` 则需要值也一致
        :param use_id: 为 `True` 则使用 `is` 判断 `val`，否则调用 `==` 判断 `val`
        :return: 是否通过检查
        """
        # pylint: disable=consider-iterating-dictionary
        if namespace not in self.__flag_mixin_flags__.keys():
            return False
        if flag not in self.__flag_mixin_flags__[namespace].keys():
            return False
        flag = self.__flag_mixin_flags__[namespace][flag]

        if not check_val:
            return True
        if use_id:
            return flag is val
        return cast(bool, flag == val)

    async def flag_wait(
        self,
        namespace: Any,
        flag: Any,
        val: Any = None,
        wait_val: bool = True,
        use_id: bool = False,
    ) -> None:
        """等待标记

        注：不同的对象并不共享 `namespace`，`namespace` 只适用于单个对象

        :param namespace: 命名空间
        :param flag: 标记
        :param val: 标记值
        :param wait_val: 为 `True` 则需要值也一致
        :param use_id: 为 `True` 则使用 `is` 判断 `val`，否则调用 `==` 判断 `val`
        :return: Future 对象
        """
        if self.flag_check(namespace, flag, val, wait_val, use_id):
            return None

        signal: Future[None] = get_running_loop().create_future()
        waitings = self.__flag_mixin_waitings__.setdefault((namespace, flag), [])
        waitings.append((val, signal, use_id, wait_val))
        await signal
        waitings = list(filter(lambda x: not x[1].done(), waitings))
        if not len(waitings):
            self.__flag_mixin_waitings__.pop((namespace, flag))


class AttrReprMixin:
    def __repr__(self) -> str:
        attrs = ", ".join(
            f"{k}={repr(v)}" for k, v in self.__dict__.items() if not k.startswith("_")
        )
        if len(attrs) >= 100:
            attrs = attrs[:100] + "..."
        return f"{self.__class__.__name__}({attrs})"


class LocateMixin:
    def __new__(cls, *_args: Any, **_kwargs: Any) -> Self:
        obj = super().__new__(cls)
        obj.__obj_location__ = obj.__location_init__()  # type: ignore[attr-defined]
        return obj

    def __init__(self) -> None:
        self.__obj_location__: tuple[str, str, int]

    @staticmethod
    def __location_init__() -> tuple[str, str, int]:
        frame = inspect.currentframe()
        while frame:
            if frame.f_code.co_name == "<module>":
                return (
                    frame.f_globals["__name__"],
                    frame.f_globals["__file__"],
                    frame.f_lineno,
                )
            frame = frame.f_back

        return ("<unknown module>", "<unknown file>", -1)

    @property
    def __obj_module__(self) -> str:
        return self.__obj_location__[0]

    @property
    def __obj_file__(self) -> str:
        return self.__obj_location__[1]

    @property
    def __obj_line__(self) -> int:
        return self.__obj_location__[2]


class HookMixin(Generic[HookEnumT]):
    def __init__(self, hook_type: type[HookEnumT], hook_tag: str | None = None):
        super().__init__()
        self._hook_bus = HookBus[HookEnumT](hook_type, hook_tag)
        self.__repeatable_hook_types__: set[HookEnumT] = set()

    def __mark_repeatable_hooks__(self, *types: HookEnumT) -> None:
        for t in types:
            self.__repeatable_hook_types__.add(t)

    def on(
        self, *periods: HookEnumT
    ) -> Callable[[SyncOrAsyncCallable[P, None]], AsyncCallable[P, None]]:
        """注册一个 hook

        :param periods: 要绑定的 hook 类型
        :return: 装饰器
        """

        def hook_register_wrapped(
            func: SyncOrAsyncCallable[P, None]
        ) -> AsyncCallable[P, None]:
            f = to_async(func)
            for type in periods:
                once = type not in self.__repeatable_hook_types__
                self._hook_bus.register(type, func, once)
            return f

        return hook_register_wrapped



from ..ctx import BotCtx as _BotCtx
from .base import CLI_RUNTIME, LAST_EXIT_SIGNAL, Bot, BotExitSignal, BotLifeSpan


def get_bot() -> Bot:
    """获得当前上下文中的 bot 对象

    :return: bot 对象
    """
    return _BotCtx().get()



import inspect
from abc import ABCMeta

from typing_extensions import Any, Callable, cast

from .base import T


class BetterABCMeta(ABCMeta):
    """更好的抽象元类，兼容 `ABCMeta` 的所有功能，但是额外支持 :func:`abstractattr`"""

    class DummyAttribute: ...

    def __call__(cls: type[T], *args: Any, **kwargs: Any) -> T:
        instance = ABCMeta.__call__(cls, *args, **kwargs)
        lack_attrs = set()
        for name in dir(instance):
            try:
                attr = getattr(instance, name)
            except Exception:
                if not isinstance(getattr(instance.__class__, name), property):
                    raise

            if getattr(attr, "__is_abstract_attribute__", False):
                lack_attrs.add(name)
            if inspect.iscoroutine(attr):
                attr.close()

        if lack_attrs:
            raise NotImplementedError(
                "Can't instantiate abstract class {} with"
                " abstract attributes: {}".format(cls.__name__, ", ".join(lack_attrs))
            )
        return cast(T, instance)


class BetterABC(metaclass=BetterABCMeta):
    """更好的抽象类，兼容 `ABC` 的所有功能，但是额外支持 :func:`abstractattr`"""

    __slots__ = ()


class SingletonMeta(type):
    """单例元类

    相比单例装饰器，可以自动保证所有子类都为单例
    """

    __instances__: dict[type, Any] = {}

    def __call__(cls: type[T], *args: Any, **kwargs: Any) -> T:
        if cls not in SingletonMeta.__instances__:
            SingletonMeta.__instances__[cls] = cast(
                T, super(SingletonMeta, cls).__call__(*args, **kwargs)  # type: ignore[misc]
            )
        return cast(T, SingletonMeta.__instances__[cls])


class SingletonBetterABCMeta(BetterABCMeta):
    """单例抽象元类

    相比普通的抽象元类，还可以自动保证所有子类都为单例
    """

    __instances__: dict[type, Any] = {}

    def __call__(cls: type[T], *args: Any, **kwargs: Any) -> T:
        mcls = SingletonBetterABCMeta
        if cls not in mcls.__instances__:
            mcls.__instances__[cls] = BetterABCMeta.__call__(cls, *args, **kwargs)
        return cast(T, mcls.__instances__[cls])


def abstractattr(obj: Callable[[Any], T] | None = None) -> T:
    """抽象属性

    与 `abstractproperty` 相比更灵活，`abstractattr` 不关心你以何种形式定义属性。只要子类在实例化后，该属性存在，即认为合法。

    但注意它必须与 :class:`BetterABC` 或 :class:`BetterABCMeta` 或 :class:`.SingletonBetterABCMeta` 配合使用

    这意味着可以在类层级、实例层级定义属性，或使用 `property` 定义属性：

    .. code:: python

        class A(BetterABC):
            foo: int = abstractattr()  # 声明为抽象属性

            # 或者使用装饰器的形式声明，这与上面是等价的
            @abstractattr
            def bar(self) -> int: ...

        # 以下实现都是合法的：

        class B(A):
            foo = 2
            bar = 4

        class C(A):
            foo = 3
            def __init__(self) -> None:
                self.bar = 5

        class D(A):
            def __init__(self) -> None:
                self.foo = 8

            @property
            def bar(self) -> int:
                return self.foo + 10
    """
    _obj = cast(Any, obj)
    if obj is None:
        _obj = BetterABCMeta.DummyAttribute()
    setattr(_obj, "__is_abstract_attribute__", True)
    return cast(T, _obj)



from __future__ import annotations

import asyncio
import inspect
from asyncio import Condition, Future, Lock
from contextlib import _AsyncGeneratorContextManager, asynccontextmanager

from typing_extensions import Any, AsyncGenerator

from ..adapter.model import Event
from ..ctx import FlowCtx, SessionCtx
from ..exceptions import SessionRuleLacked, SessionStateFailed
from ..handle.base import EventCompletion, stop
from ..typ.base import SyncOrAsyncCallable
from .option import CompareInfo, Rule

_SESSION_CTX = SessionCtx()


class SessionState:
    def __init__(self, session: "Session") -> None:
        self.session = session

    async def work(self, completion: EventCompletion) -> None:
        raise SessionStateFailed(self.__class__.__name__, SessionState.work.__name__)

    async def rest(self) -> None:
        raise SessionStateFailed(self.__class__.__name__, SessionState.rest.__name__)

    async def suspend(self, timeout: float | None) -> bool:
        raise SessionStateFailed(self.__class__.__name__, SessionState.suspend.__name__)

    async def wakeup(self, completion: EventCompletion | None) -> None:
        raise SessionStateFailed(self.__class__.__name__, SessionState.wakeup.__name__)

    async def expire(self) -> None:
        raise SessionStateFailed(self.__class__.__name__, SessionState.expire.__name__)


class SpareSessionState(SessionState):
    async def work(self, completion: EventCompletion) -> None:
        self.session._completions.add(completion)
        self.session.event = completion.event
        self.session.__to_state__(WorkingSessionState)


class WorkingSessionState(SessionState):
    async def rest(self) -> None:
        if self.session.rule is None:
            raise SessionRuleLacked("缺少会话规则，会话无法从“运行态”转为“空闲态”")

        cond = self.session._refresh_cond
        self.session.__to_state__(SpareSessionState)
        async with cond:
            cond.notify()

    async def suspend(self, timeout: float | None) -> bool:
        self.session.set_completed()

        if self.session.rule is None:
            raise SessionRuleLacked("缺少会话规则，会话无法从“运行态”转为“挂起态”")

        cond = self.session._refresh_cond
        self.session.__to_state__(SuspendSessionState)
        async with cond:
            cond.notify()

        async with self.session._wakeup_cond:
            if timeout is None:
                await self.session._wakeup_cond.wait()
                return True

            try:
                await asyncio.wait_for(self.session._wakeup_cond.wait(), timeout=timeout)
                return True

            except asyncio.TimeoutError:
                if self.session.__is_state__(WorkingSessionState):
                    return True
                self.session.__to_state__(WorkingSessionState)
                return False

    async def expire(self) -> None:
        self.session.__to_state__(ExpireSessionState)
        if self.session.rule is not None:
            cond = self.session._refresh_cond
            async with cond:
                cond.notify()

        self.session.set_completed()


class SuspendSessionState(SessionState):

    async def wakeup(self, completion: EventCompletion | None) -> None:
        if self.session.__is_state__(WorkingSessionState):
            return

        if completion is not None:
            self.session._completions.add(completion)
            self.session.event = completion.event
        cond = self.session._wakeup_cond
        self.session.__to_state__(WorkingSessionState)
        async with cond:
            cond.notify()


class ExpireSessionState(SessionState): ...


class SessionStore(dict[str, Any]):
    """会话存储，生命周期伴随会话对象"""

    def set(self, key: str, value: Any) -> None:
        self[key] = value


class Session:
    """会话

    :ivar SessionStore store: 当前会话上下文的会话存储
    :ivar Rule rule: 当前会话上下文的会话规则
    """

    __instances__: dict[Rule, set["Session"]] = {}
    __instance_locks__: dict[Rule, Lock] = {}
    __cls_lock__ = Lock()

    def __init__(
        self,
        rule: Rule | None,
        first_completion: EventCompletion,
        keep: bool = False,
        auto_complete: bool = True,
    ) -> None:
        self.store: SessionStore = SessionStore()
        self.event = first_completion.event
        self.rule = rule
        self.auto_complete = auto_complete

        self._completions: set[EventCompletion] = set()
        self._completions.add(first_completion)
        self._refresh_cond = Condition()
        self._wakeup_cond = Condition()
        self._keep = keep
        self._state: SessionState = WorkingSessionState(self)

    def stop_keep(self) -> None:
        self._keep = False

    def set_completed(self, event: Event | None = None) -> None:
        if event is None:
            for c in self._completions:
                c.completed.set_result(None)
            self._completions.clear()
            return

        comps = filter(lambda c: c.event is event, self._completions)
        for c in comps:
            c.completed.set_result(None)
            self._completions.remove(c)

    def get_incompletions(self) -> list[tuple[Event, Future]]:
        return [
            (c.event, c.completed) for c in self._completions if not c.completed.done()
        ]

    def __to_state__(self, state_class: type[SessionState]) -> None:
        self._state = state_class(self)

    def __is_state__(self, state_class: type[SessionState]) -> bool:
        return isinstance(self._state, state_class)

    async def __work__(self, completion: EventCompletion) -> None:
        await self._state.work(completion)

    async def __rest__(self) -> None:
        await self._state.rest()

    async def __suspend__(self, timeout: float | None = None) -> bool:
        return await self._state.suspend(timeout)

    async def __wakeup__(self, completion: EventCompletion | None) -> None:
        await self._state.wakeup(completion)

    async def __expire__(self) -> None:
        await self._state.expire()

    @classmethod
    async def get(
        cls,
        completion: EventCompletion,
        rule: Rule | None = None,
        wait: bool = True,
        nowait_cb: SyncOrAsyncCallable[[], None] | None = None,
        keep: bool = False,
        auto_complete: bool = True,
    ) -> Session | None:
        event = completion.event
        if rule is None:
            return Session(
                rule=None,
                first_completion=completion,
                keep=keep,
                auto_complete=auto_complete,
            )

        async with cls.__cls_lock__:
            cls.__instance_locks__.setdefault(rule, Lock())

        async with cls.__instance_locks__[rule]:
            try:
                _set = cls.__instances__.setdefault(rule, set())

                suspends = filter(lambda s: s.__is_state__(SuspendSessionState), _set)
                for session in suspends:
                    if await rule.compare_with(
                        CompareInfo(session, session.event, event)
                    ):
                        await session.__wakeup__(completion)
                        return None

                spares = filter(lambda s: s.__is_state__(SpareSessionState), _set)
                for session in spares:
                    if await rule.compare_with(
                        CompareInfo(session, session.event, event)
                    ):
                        await session.__work__(completion)
                        session._keep = keep
                        return session

                workings = filter(lambda s: s.__is_state__(WorkingSessionState), _set)
                for session in workings:
                    if not await rule.compare_with(
                        CompareInfo(session, session.event, event)
                    ):
                        continue

                    if not wait:
                        if nowait_cb is not None:
                            ret = nowait_cb()
                            if inspect.isawaitable(ret):
                                await ret
                        completion.completed.set_result(None)
                        return None

                    cond = session._refresh_cond
                    async with cond:
                        await cond.wait()

                        if session.__is_state__(ExpireSessionState):
                            pass

                        elif session.__is_state__(SuspendSessionState):
                            await session.__wakeup__(completion)
                            return None

                        else:
                            await session.__work__(completion)
                            session._keep = keep
                            return session

                session = Session(
                    rule=rule,
                    first_completion=completion,
                    keep=keep,
                    auto_complete=auto_complete,
                )
                Session.__instances__[rule].add(session)
                return session

            finally:
                expires = tuple(
                    filter(lambda s: s.__is_state__(ExpireSessionState), _set)
                )
                for session in expires:
                    Session.__instances__[rule].remove(session)

    @classmethod
    @asynccontextmanager
    async def enter(
        cls,
        rule: Rule,
        wait: bool = True,
        nowait_cb: SyncOrAsyncCallable[[], None] | None = None,
        keep: bool = False,
        auto_complete: bool = True,
    ) -> AsyncGenerator[Session, None]:
        flow_ctx = FlowCtx()
        completion = flow_ctx.get_completion()
        completion.under_session = True

        session = await cls.get(
            completion,
            rule=rule,
            wait=wait,
            nowait_cb=nowait_cb,
            keep=keep,
            auto_complete=auto_complete,
        )
        if session is None:
            await stop()

        with _SESSION_CTX.unfold(session):
            try:
                yield session
            except asyncio.CancelledError:
                if session.__is_state__(SuspendSessionState):
                    await session.__wakeup__(completion=None)
            finally:
                if session._keep:
                    await session.__rest__()
                else:
                    await session.__expire__()


async def suspend(timeout: float | None = None) -> bool:
    """挂起当前会话

    :param timeout: 挂起后再唤醒的超时时间, 为空则永不超时
    :return: 如果为 `False` 则表明唤醒超时
    """
    return await _SESSION_CTX.get().__suspend__(timeout)


def enter_session(
    rule: Rule,
    wait: bool = True,
    nowait_cb: SyncOrAsyncCallable[[], None] | None = None,
    keep: bool = False,
    auto_complete: bool = True,
) -> _AsyncGeneratorContextManager[Session]:
    """上下文管理器，提供一个会话上下文，在此上下文中可使用会话的高级特性

    :param rule: 会话规则
    :param wait: 当出现会话冲突时，是否需要等待
    :param nowait_cb: 指定了 `wait=False` 后，会话冲突时执行的回调
    :param keep: 会话在退出会话上下文后是否继续保持
    :param auto_complete: 当前会话挂起后，事件是否自动向更低优先级传播
    :yield: 会话对象
    """
    return Session.enter(rule, wait, nowait_cb, keep, auto_complete)



from __future__ import annotations

import asyncio
from abc import abstractmethod
from asyncio import create_task
from contextlib import AsyncExitStack, _GeneratorContextManager, asynccontextmanager
from enum import Enum
from os import PathLike

from typing_extensions import (
    TYPE_CHECKING,
    AsyncGenerator,
    Callable,
    Generic,
    Iterable,
    LiteralString,
    NoReturn,
    Self,
    Sequence,
    TypeVar,
    cast,
    final,
)

from ..ctx import EventOrigin, FlowCtx, LoggerCtx, OutSrcFilterCtx
from ..exceptions import AdapterError
from ..io.base import (
    AbstractInSource,
    AbstractOutSource,
    EchoPacketT,
    InPacketT,
    InSourceT,
    OutPacketT,
    OutSourceT,
)
from ..log.base import LogLevel
from ..mixin import HookMixin
from ..typ.cls import BetterABC
from .content import Content
from .model import ActionHandle, ActionT, EchoT, Event, EventT

if TYPE_CHECKING:
    from ..bot.dispatch import Dispatcher


_OUT_SRC_FILTER_CTX = OutSrcFilterCtx()
_FLOW_CTX = FlowCtx()


class AbstractEventFactory(BetterABC, Generic[InPacketT, EventT]):
    """抽象事件工厂类"""

    @abstractmethod
    async def create(self, packet: InPacketT) -> EventT:
        """将 :class:`.InPacket` 对象转换为 :class:`.Event` 对象的方法

        :param packet: 输入包
        :return: 事件
        """
        raise NotImplementedError


EventFactoryT = TypeVar("EventFactoryT", bound=AbstractEventFactory)


class AbstractOutputFactory(BetterABC, Generic[OutPacketT, ActionT]):
    """抽象输出工厂类"""

    @abstractmethod
    async def create(self, action: ActionT) -> OutPacketT:
        """将 :class:`.Action` 对象转换为 :class:`.OutPacket` 对象的方法

        :param packet: 行为
        :return: 输出包
        """
        raise NotImplementedError


OutputFactoryT = TypeVar("OutputFactoryT", bound=AbstractOutputFactory)


class AbstractEchoFactory(BetterABC, Generic[EchoPacketT, EchoT]):
    """抽象回应工厂类"""

    @abstractmethod
    async def create(self, packet: EchoPacketT) -> EchoT | None:
        """将 :class:`.EchoPacket` 对象转换为 :class:`.Echo` 对象的方法

        :param packet: 回应包
        :return: 回应或空值
        """
        raise NotImplementedError


EchoFactoryT = TypeVar("EchoFactoryT", bound=AbstractEchoFactory)


class AdapterLifeSpan(Enum):
    """适配器生命周期阶段的枚举"""

    BEFORE_EVENT_HANDLE = "beh"
    BEFORE_ACTION_EXEC = "bae"
    STARTED = "sta"
    STOPPED = "sto"


class Adapter(
    HookMixin[AdapterLifeSpan],
    Generic[
        EventFactoryT,
        OutputFactoryT,
        EchoFactoryT,
        ActionT,
        InSourceT,
        OutSourceT,
    ],
    BetterABC,
):
    """适配器基类

    :ivar LiteralString protocol: 适配器所使用的协议
    """

    # pylint: disable=duplicate-code
    # pylint: disable=unused-argument

    def __init__(
        self,
        protocol: LiteralString,
        event_factory: EventFactoryT,
        output_factory: OutputFactoryT,
        echo_factory: EchoFactoryT,
    ) -> None:
        super().__init__(hook_type=AdapterLifeSpan, hook_tag=protocol)

        self.protocol = protocol
        self.in_srcs: set[InSourceT] = set()
        self.out_srcs: set[OutSourceT] = set()
        self.dispatcher: "Dispatcher"
        self._event_factory = event_factory
        self._output_factory = output_factory
        self._echo_factory = echo_factory

        self._inited = False
        self.__mark_repeatable_hooks__(
            AdapterLifeSpan.BEFORE_EVENT_HANDLE, AdapterLifeSpan.BEFORE_ACTION_EXEC
        )

    @final
    def get_isrcs(self, filter: Callable[[InSourceT], bool]) -> set[InSourceT]:
        """获取与当前适配器匹配的所有输入源

        :param filter: 过滤函数，为 `True` 时保留输入源
        :return: 输入源的集合
        """
        return set(src for src in self.in_srcs if filter(src))

    @final
    def get_osrcs(self, filter: Callable[[OutSourceT], bool]) -> set[OutSourceT]:
        """获取与当前适配器匹配的所有输出源

        :param filter: 过滤函数，为 `True` 时保留输出源
        :return: 输出源的集合
        """
        return set(src for src in self.out_srcs if filter(src))

    @final
    async def __adapter_input_loop__(self, src: InSourceT) -> NoReturn:
        logger = LoggerCtx().get()
        while True:
            try:
                packet = await src.input()
                event: Event = await self._event_factory.create(packet)
                EventOrigin.set_origin(event, EventOrigin(self, src))
                await self._hook_bus.emit(
                    AdapterLifeSpan.BEFORE_EVENT_HANDLE, True, args=(event,)
                )
                self.dispatcher.broadcast(event)
            except Exception:
                logger.exception(f"适配器 {self} 处理输入与分发事件时发生异常")
                logger.generic_obj(
                    "异常点局部变量：",
                    {
                        "in_factory": self._event_factory,
                        "dispatcher": self.dispatcher,
                    }
                    | locals(),
                    level=LogLevel.ERROR,
                )

    @asynccontextmanager
    @final
    async def __adapter_launch__(self) -> AsyncGenerator[Self, None]:
        if self._inited:
            raise AdapterError(f"适配器 {self} 已在运行，不能重复启动")

        try:
            async with AsyncExitStack() as stack:
                out_src_ts = tuple(
                    create_task(stack.enter_async_context(src)) for src in self.out_srcs
                )
                if len(out_src_ts):
                    await asyncio.wait(out_src_ts)

                in_src_ts = tuple(
                    create_task(stack.enter_async_context(src))
                    for src in self.in_srcs
                    if not src.opened()
                )
                if len(in_src_ts):
                    await asyncio.wait(in_src_ts)

                for src in self.in_srcs:
                    create_task(self.__adapter_input_loop__(src))

                self._inited = True
                await self._hook_bus.emit(AdapterLifeSpan.STARTED)
                yield self

        finally:
            if self._inited:
                await self._hook_bus.emit(AdapterLifeSpan.STOPPED, True)

    @final
    def filter_out(
        self, filter: Callable[[OutSourceT], bool]
    ) -> _GeneratorContextManager[Callable[[OutSourceT], bool]]:
        """上下文管理器，提供由 `filter` 控制输出的输出上下文

        :param filter: 过滤函数，为 `True` 时允许该输出源输出
        """
        return _OUT_SRC_FILTER_CTX.unfold(filter)

    async def call_output(self, action: ActionT) -> tuple[ActionHandle, ...]:
        """输出行为，并返回各个输出源返回的 :class:`.ActionHandle` 组成的元组

        适配器开发者的适配器子类可以重写此方法，以实现自定义功能

        但子类实现中必须调用原始方法： `super().call_output(...)`

        :param action: 行为
        :return: :class:`.ActionHandle` 元组
        """
        osrcs: Iterable[OutSourceT]
        filter = _OUT_SRC_FILTER_CTX.try_get()
        cur_isrc: AbstractInSource | None

        if event := _FLOW_CTX.try_get_event():
            cur_isrc = EventOrigin.get_origin(event).in_src
        else:
            cur_isrc = None

        if filter is not None:
            osrcs = (osrc for osrc in self.out_srcs if filter(osrc))
        elif isinstance(cur_isrc, AbstractOutSource):
            osrcs = (cast(OutSourceT, cur_isrc),)
        else:
            osrcs = self.out_srcs if len(self.out_srcs) else ()

        await self._hook_bus.emit(
            AdapterLifeSpan.BEFORE_ACTION_EXEC, True, args=(action,)
        )
        return tuple(
            ActionHandle(action, osrc, self._output_factory, self._echo_factory)
            for osrc in osrcs
        )

    @abstractmethod
    async def __send_text__(self, text: str) -> tuple[ActionHandle, ...]:
        """输出文本

        抽象方法。所有适配器子类应该实现此方法

        :param text: 文本
        :return: :class:`.ActionHandle` 元组
        """
        raise NotImplementedError

    async def __send_media__(
        self,
        name: str,
        raw: bytes | None = None,
        url: str | None = None,
        mimetype: str | None = None,
    ) -> tuple[ActionHandle, ...]:
        """输出多媒体内容

        建议所有适配器子类重写此方法，否则回退到基类实现：仅使用 :func:`send_text` 输出相关提示信息

        :param name: 多媒体内容的名称
        :param raw: 多媒体内容的二进制内容
        :param url: 多媒体内容的网络地址（和 `raw` 参数二选一）
        :param mimetype: 多媒体内容的 mimetype，为空则根据 `name` 自动检测
        :return: :class:`.ActionHandle` 元组
        """
        return await self.__send_text__(
            f"[melobot media: {name if url is None else name + ' at ' + url}]"
        )

    async def __send_image__(
        self,
        name: str,
        raw: bytes | None = None,
        url: str | None = None,
        mimetype: str | None = None,
    ) -> tuple[ActionHandle, ...]:
        """输出图像内容

        建议所有适配器子类重写此方法，否则回退到基类实现：仅使用 :func:`send_text` 输出相关提示信息

        :param name: 图像内容的名称
        :param raw: 图像内容的二进制内容
        :param url: 图像内容的网络地址（和 `raw` 参数二选一）
        :param mimetype: 图像内容的 mimetype，为空则根据 `name` 自动检测
        :return: :class:`.ActionHandle` 元组
        """
        return await self.__send_text__(
            f"[melobot image: {name if url is None else name + ' at ' + url}]"
        )

    async def __send_audio__(
        self,
        name: str,
        raw: bytes | None = None,
        url: str | None = None,
        mimetype: str | None = None,
    ) -> tuple[ActionHandle, ...]:
        """输出音频内容

        建议所有适配器子类重写此方法，否则回退到基类实现：仅使用 :func:`send_text` 输出相关提示信息

        :param name: 音频内容的名称
        :param raw: 音频内容的二进制内容
        :param url: 音频内容的网络地址（和 `raw` 参数二选一）
        :param mimetype: 音频内容的 mimetype，为空则根据 `name` 自动检测
        :return: :class:`.ActionHandle` 元组
        """
        return await self.__send_text__(
            f"[melobot audio: {name if url is None else name + ' at ' + url}]"
        )

    async def __send_voice__(
        self,
        name: str,
        raw: bytes | None = None,
        url: str | None = None,
        mimetype: str | None = None,
    ) -> tuple[ActionHandle, ...]:
        """输出语音内容

        建议所有适配器子类重写此方法，否则回退到基类实现：仅使用 :func:`send_text` 输出相关提示信息

        :param name: 语音内容的名称
        :param raw: 语音内容的二进制内容
        :param url: 语音内容的网络地址（和 `raw` 参数二选一）
        :param mimetype: 语音内容的 mimetype，为空则根据 `name` 自动检测
        :return: :class:`.ActionHandle` 元组
        """
        return await self.__send_text__(
            f"[melobot voice: {name if url is None else name + ' at ' + url}]"
        )

    async def __send_video__(
        self,
        name: str,
        raw: bytes | None = None,
        url: str | None = None,
        mimetype: str | None = None,
    ) -> tuple[ActionHandle, ...]:
        """输出视频内容

        建议所有适配器子类重写此方法，否则回退到基类实现：仅使用 :func:`send_text` 输出相关提示信息

        :param name: 视频内容的名称
        :param raw: 视频内容的二进制内容
        :param url: 视频内容的网络地址（和 `raw` 参数二选一）
        :param mimetype: 视频内容的 mimetype，为空则根据 `name` 自动检测
        :return: :class:`.ActionHandle` 元组
        """
        return await self.__send_text__(
            f"[melobot video: {name if url is None else name + ' at ' + url}]"
        )

    async def __send_file__(
        self, name: str, path: str | PathLike[str]
    ) -> tuple[ActionHandle, ...]:
        """输出文件

        建议所有适配器子类重写此方法，否则回退到基类实现：仅使用 :func:`send_text` 输出相关提示信息

        :param name: 文件名
        :param path: 文件路径
        :return: :class:`.ActionHandle` 元组
        """
        return await self.__send_text__(f"[melobot file: {name} at {path}]")

    async def __send_refer__(
        self, event: Event, contents: Sequence[Content] | None = None
    ) -> tuple[ActionHandle, ...]:
        """输出对过往事件的引用

        建议所有适配器子类重写此方法，否则回退到基类实现：仅使用 :func:`send_text` 输出相关提示信息

        :param event: 过往的事件对象
        :param contents: 附加的通用内容序列
        :return: :class:`.ActionHandle` 元组
        """
        return await self.__send_text__(
            f"[melobot refer: {event.__class__.__name__}({event.id})]"
        )

    async def __send_resource__(self, name: str, url: str) -> tuple[ActionHandle, ...]:
        """输出网络资源

        建议所有适配器子类重写此方法，否则回退到基类实现：仅使用 :func:`send_text` 输出相关提示信息

        :param name: 网络资源名称
        :param url: 网络资源的 url
        :return: :class:`.ActionHandle` 元组
        """
        return await self.__send_text__(f"[melobot resource: {name} at {url}]")



import sys
import zipimport

if sys.version_info < (3, 11):
    from importlib._bootstrap_external import _NamespaceLoader
else:
    from importlib.machinery import NamespaceLoader as _NamespaceLoader

from importlib.abc import FileLoader, Loader, MetaPathFinder
from importlib.machinery import (
    BYTECODE_SUFFIXES,
    EXTENSION_SUFFIXES,
    SOURCE_SUFFIXES,
    ExtensionFileLoader,
    ModuleSpec,
)
from importlib.machinery import PathFinder as _PathFinder
from importlib.machinery import SourceFileLoader, SourcelessFileLoader, all_suffixes
from importlib.util import module_from_spec, spec_from_file_location
from os import PathLike
from pathlib import Path
from types import ModuleType

from typing_extensions import Any, Sequence, cast

from .exceptions import DynamicImpError
from .utils.common import singleton

ALL_EXTS = tuple(all_suffixes())
EMPTY_PKG_TAG = "__melobot_namespace_pkg__"
ZIP_MODULE_TAG = "__melobot_zip_module__"


def _get_file_loaders() -> list[tuple[type[Loader], list[str]]]:
    extensions = (ExtensionFileLoader, EXTENSION_SUFFIXES)
    source = (SourceFileLoader, SOURCE_SUFFIXES)
    bytecode = (SourcelessFileLoader, BYTECODE_SUFFIXES)
    return [extensions, source, bytecode]


class _NestedQuickExit(BaseException): ...


@singleton
class SpecFinder(MetaPathFinder):
    def find_spec(
        self,
        fullname: str,
        paths: Sequence[str] | None,
        target: ModuleType | None = None,
        sys_cache: bool = True,
        load_cache: bool = True,
        pre_sys_len: int = -1,
        pre_cache_len: int = -1,
    ) -> ModuleSpec | None:
        if paths is None:
            paths = sys.path
        if "." in fullname:
            *_, name = fullname.split(".")
        else:
            name = fullname

        mod_path: Path | None = None
        submod_locs: list[str] | None = None

        # 模块查找的优先级，遵循 PEP420: https://peps.python.org/pep-0420/#specification
        try:
            for entry in paths:
                entry_path = Path(entry)
                dir_path = entry_path.joinpath(name)
                pkg_init_path = dir_path.joinpath("__init__.py")

                # 带有 __init__.py 的包优先
                if pkg_init_path.exists():
                    mod_path = pkg_init_path
                    submod_locs = [str(dir_path.resolve())]
                    raise _NestedQuickExit

                # 其次是各种可加载的文件
                for ext in ALL_EXTS:
                    _mod_path = entry_path.joinpath(f"{name}{ext}")
                    if _mod_path.exists():
                        mod_path = _mod_path
                        submod_locs = None
                        raise _NestedQuickExit

                # 再次是 zip 文件导入
                if entry_path.suffix == ".zip" and entry_path.exists():
                    zip_importer = zipimport.zipimporter(str(entry_path))
                    spec = zip_importer.find_spec(fullname, target)
                    if spec is not None:
                        assert spec.origin is not None and spec.origin != "", (
                            f"zip file from {entry_path}, module named {fullname} from {target}, "
                            "failed to get spec origin"
                        )
                        assert spec.loader is not None, (
                            f"zip file from {entry_path}, module named {fullname} from {target}, "
                            "spec has no loader"
                        )
                        spec.loader = ModuleLoader(
                            fullname,
                            Path(spec.origin).resolve(),
                            sys_cache,
                            load_cache,
                            pre_sys_len,
                            pre_cache_len,
                            spec.loader,
                        )
                        setattr(spec, ZIP_MODULE_TAG, True)
                        return spec

                # 没有 __init__.py 的包最后查找，spec 设置为与内置导入兼容的命名空间包格式
                if dir_path.exists() and dir_path.is_dir():
                    dir_path_str = str(dir_path.resolve())
                    loader = cast(
                        Loader,
                        _NamespaceLoader(
                            fullname,
                            [dir_path_str],
                            _PathFinder._get_spec,  # type: ignore[attr-defined]
                        ),
                    )
                    spec = spec_from_file_location(
                        fullname,
                        dir_path_str,
                        loader=ModuleLoader(
                            fullname,
                            dir_path,
                            sys_cache,
                            load_cache,
                            pre_sys_len,
                            pre_cache_len,
                            loader,
                        ),
                        submodule_search_locations=loader._path,  # type: ignore[attr-defined]
                    )
                    assert (
                        spec is not None
                    ), f"package from {dir_path} without __init__.py create spec failed"
                    spec.has_location = False
                    spec.origin = None
                    setattr(spec, EMPTY_PKG_TAG, True)
                    return spec

        except _NestedQuickExit:
            pass

        if mod_path is None and submod_locs is None:
            return None

        mod_path = cast(Path, mod_path).resolve()
        return spec_from_file_location(
            fullname,
            str(mod_path),
            loader=ModuleLoader(
                fullname, mod_path, sys_cache, load_cache, pre_sys_len, pre_cache_len
            ),
            submodule_search_locations=submod_locs,
        )


@singleton
class ModuleCacher:
    def __init__(self) -> None:
        self._caches: dict[Path, ModuleType] = {}
        self.clear_cache()

    def get_len(self) -> int:
        return len(self._caches)

    def has_cache(self, mod: ModuleType) -> bool:
        return mod in self._caches.values()

    def get_cache(self, path: Path) -> ModuleType | None:
        # 对应有 __init__.py 的包模块
        if path.parts[-1] == "__init__.py":
            path = path.parent
        return self._caches.get(path)

    def set_cache(self, name: str, mod: ModuleType) -> None:
        if (
            mod in self._caches.values()
            or name in sys.stdlib_module_names
            or not hasattr(mod, "__file__")
        ):
            return

        # __file__ 存在且不为空，可能包或任意可被加载的文件
        if mod.__file__ is not None:
            fp = Path(mod.__file__)
            if fp.parts[-1] == "__init__.py":
                self._caches[fp.parent] = mod
            else:
                self._caches[fp] = mod
        # 若 __file__ 为空则有 __path__，对应无 __init__.py 的包
        else:
            self._caches[Path(mod.__path__[0])] = mod

    def rm_lastn(self, n: int) -> None:
        iter = reversed(self._caches.keys())
        rm_paths: list[Path] = []
        for _ in range(n):
            rm_paths.append(next(iter))
        for p in rm_paths:
            self._caches.pop(p)

    def clear_cache(self) -> None:
        self._caches.clear()
        for name, mod in sys.modules.items():
            self.set_cache(name, mod)


class ModuleLoader(Loader):
    def __init__(
        self,
        fullname: str,
        fp: Path,
        sys_cache: bool,
        load_cache: bool,
        pre_sys_len: int = -1,
        pre_cache_len: int = -1,
        inner_loader: Loader | None = None,
    ) -> None:
        super().__init__()
        # 避免在 self.__getattr__() 反射时重名
        self.melobot_cacher = ModuleCacher()
        self.melobot_fullname = fullname
        self.melobot_fp = fp
        self.melobot_sys_cache = sys_cache
        self.melobot_load_cache = load_cache
        self.melobot_pre_sys_len = pre_sys_len
        self.melobot_pre_cache_len = pre_cache_len

        self.melobot_inner_loader: Loader | None = inner_loader
        if inner_loader is not None:
            return

        for loader_cls, suffixes in _get_file_loaders():
            if str(fp).endswith(tuple(suffixes)):
                loader_cls = cast(type[FileLoader], loader_cls)
                loader = loader_cls(fullname, str(fp))  # pylint: disable=not-callable
                self.melobot_inner_loader = loader
                break

    def create_module(self, spec: ModuleSpec) -> ModuleType | None:
        mod = None
        if self.melobot_load_cache:
            mod = self.melobot_cacher.get_cache(self.melobot_fp)
        if mod is None and self.melobot_inner_loader is not None:
            mod = self.melobot_inner_loader.create_module(spec)
        return mod

    def exec_module(self, mod: ModuleType) -> None:
        if (
            not self.melobot_cacher.has_cache(mod)
            and self.melobot_inner_loader is not None
        ):
            # 遵循先记录原则，防止 exec_module 发起的某些递归导入出现错误
            if self.melobot_sys_cache:
                sys.modules[self.melobot_fullname] = mod

            try:
                self.melobot_inner_loader.exec_module(mod)
                # 设置为与内置导入机制兼容的模式
                if hasattr(mod.__spec__, EMPTY_PKG_TAG):
                    mod.__file__ = None
            except BaseException:
                try:
                    del sys.modules[self.melobot_fullname]
                except KeyError:
                    pass
                raise
        # 若 inner_loader 为空，则是纯粹的命名空间包（没有 __init__.py 的包模块）
        # 也就不需要任何实质性的 exec_module 过程

        if self.melobot_load_cache:
            self.melobot_cacher.set_cache(self.melobot_fullname, mod)

        if not self.melobot_load_cache:
            diff = self.melobot_cacher.get_len() - self.melobot_pre_cache_len
            if diff > 0:
                self.melobot_cacher.rm_lastn(diff)

        if not self.melobot_sys_cache:
            diff = len(sys.modules) - self.melobot_pre_sys_len
            if diff > 0:
                iter = reversed(sys.modules.keys())
                rm_names: list[str] = []
                for _ in range(diff):
                    rm_names.append(next(iter))
                for name in rm_names:
                    sys.modules.pop(name)

    def __getattr__(self, name: str) -> Any:
        # 直接使用反射，而不是更复杂的继承方案
        # inner_loader 实现了必要的 内省接口、importlib.resources 接口
        if self.melobot_inner_loader is not None:
            return getattr(self.melobot_inner_loader, name)
        raise AttributeError(
            f"'{self.__class__.__name__}' object has no attribute '{name}'"
        )


class Importer:
    @staticmethod
    def import_mod(
        name: str,
        path: str | PathLike[str] | None = None,
        sys_cache: bool = True,
        mb_cache: bool = True,
    ) -> ModuleType:
        """动态导入一个模块

        :param name: 模块名
        :param path: 在何处查找模块，为空则按照默认规则查找
        :param sys_cache: 是否加载后在 `sys.modules` 中缓存
        :param load_cache: 是否加载后在 melobot 模块缓存器中缓存
        :return: 模块
        """
        # 必须先获取，后续可能运行的递归将会影响序列长度
        pre_sys_len = len(sys.modules)
        pre_cache_len = ModuleCacher().get_len()

        if path is not None:
            try:
                sep = name.rindex(".")
                Importer.import_mod(name[:sep], Path(path).parent, True, True)
            except ValueError:
                pass

        if sys_cache and name in sys.modules:
            return sys.modules[name]

        spec = SpecFinder().find_spec(
            name,
            (str(path),) if path is not None else None,
            sys_cache=sys_cache,
            load_cache=mb_cache,
            pre_sys_len=pre_sys_len,
            pre_cache_len=pre_cache_len,
        )
        if spec is None:
            raise DynamicImpError(
                f"名为 {name} 的模块无法加载，指定的位置：{path}",
                name=name,
                path=str(path),
            )

        mod = module_from_spec(spec)
        assert (
            spec.loader is not None
        ), f"module named {name} and path from {path} has no loader"
        spec.loader.exec_module(mod)
        return mod

    @staticmethod
    def clear_cache() -> None:
        ModuleCacher().clear_cache()

    @staticmethod
    def get_cache(path: Path) -> ModuleType | None:
        return ModuleCacher().get_cache(path)


sys.meta_path.insert(0, SpecFinder())

# 兼容 pkg_resources 的资源获取操作
# 但此模块于 3.12 删除，因此前向版本不再兼容
if sys.version_info < (3, 12):
    import pkg_resources

    def _union_provider(mod: Any) -> pkg_resources.NullProvider:
        if hasattr(mod.__spec__, ZIP_MODULE_TAG):
            return pkg_resources.ZipProvider(mod)
        return pkg_resources.DefaultProvider(mod)

    pkg_resources.register_loader_type(ModuleLoader, cast(type, _union_provider))
    pkg_resources.register_namespace_handler(SpecFinder, pkg_resources.file_ns_handler)



from ._meta import MetaInfo, __version__
from .adapter import Action, Adapter, Echo, Event
from .adapter.generic import send_image, send_text
from .bot import Bot, get_bot
from .ctx import Context
from .di import Depends
from .handle import (
    Flow,
    FlowDecorator,
    FlowStore,
    node,
    on_command,
    on_contain_match,
    on_end_match,
    on_event,
    on_full_match,
    on_regex_match,
    on_start_match,
    on_text,
    rewind,
    stop,
)
from .log import GenericLogger, Logger, LogLevel, get_logger
from .plugin import AsyncShare, PluginInfo, PluginLifeSpan, PluginPlanner, SyncShare
from .session import DefaultRule, Rule, Session, SessionStore, enter_session, suspend
from .typ._enum import LogicMode



from __future__ import annotations

from pydantic import BaseModel
from typing_extensions import Any, Literal, Mapping, TypedDict, cast

from melobot.adapter import Echo as RootEcho

from ..const import ACTION_TYPE_KEY_NAME, PROTOCOL_IDENTIFIER
from .event import _GroupMessageSender, _MessageSender
from .segment import NodeSegment, Segment


class Echo(RootEcho):

    class Model(BaseModel):
        status: Literal["ok", "async", "failed"]
        retcode: int
        data: Mapping[str, Any] | list | None

    def __init__(self, **kv_pairs: Any) -> None:
        self._model = self.Model(**kv_pairs)

        _dic = kv_pairs.copy()
        self.action_type: str = _dic.pop(ACTION_TYPE_KEY_NAME)
        self.raw = _dic

        super().__init__(
            protocol=PROTOCOL_IDENTIFIER,
            ok=self._model.status == "ok",
            status=self._model.retcode,
            data=self._model.data,
        )

    def __repr__(self) -> str:
        return (
            f"{self.__class__.__name__}(status={self._model.status},"
            f" retcode={self._model.retcode}, action_type={self.action_type})"
        )

    def is_ok(self) -> bool:
        return self._model.status == "ok"

    def is_async(self) -> bool:
        return self._model.status == "async"

    def is_failed(self) -> bool:
        return self._model.status == "failed"

    @classmethod
    def resolve(cls, raw: dict[str, Any]) -> Echo:
        match raw[ACTION_TYPE_KEY_NAME]:
            case "send_private_msg" | "send_group_msg" | "send_msg":
                return SendMsgEcho(**raw)
            case "send_private_forward_msg" | "send_group_forward_msg":
                return SendForwardMsgEcho(**raw)
            case "get_msg":
                return GetMsgEcho(**raw)
            case "get_forward_msg":
                return GetForwardMsgEcho(**raw)
            case "get_login_info":
                return GetLoginInfoEcho(**raw)
            case "get_stranger_info":
                return GetStrangerInfoEcho(**raw)
            case "get_friend_list":
                return GetFriendListEcho(**raw)
            case "get_group_info":
                return GetGroupInfoEcho(**raw)
            case "get_group_list":
                return GetGroupListEcho(**raw)
            case "get_group_member_info":
                return GetGroupMemberInfoEcho(**raw)
            case "get_group_member_list":
                return GetGroupMemberListEcho(**raw)
            case "get_group_honor_info":
                return GetGroupHonorInfoEcho(**raw)
            case "get_cookies":
                return GetCookiesEcho(**raw)
            case "get_csrf_token":
                return GetCsrfTokenEcho(**raw)
            case "get_credentials":
                return GetCredentialsEcho(**raw)
            case "get_record":
                return GetRecordEcho(**raw)
            case "get_image":
                return GetImageEcho(**raw)
            case "can_send_image":
                return CanSendImageEcho(**raw)
            case "can_send_record":
                return CanSendRecordEcho(**raw)
            case "get_status":
                return GetStatusEcho(**raw)
            case "get_version_info":
                return GetVersionInfoEcho(**raw)
            case (
                "delete_msg"
                | "send_like"
                | "set_group_kick"
                | "set_group_ban"
                | "set_group_anonymous_ban"
                | "set_group_whole_ban"
                | "set_group_admin"
                | "set_group_anonymous"
                | "set_group_card"
                | "set_group_name"
                | "set_group_leave"
                | "set_group_special_title"
                | "set_friend_add_request"
                | "set_group_add_request"
                | "set_restart"
                | "clean_cache"
            ):
                return EmptyEcho(**raw)
            case _:
                return Echo(**raw)


class EmptyEcho(Echo):

    class Model(Echo.Model):
        data: None

    data: None


class _SendMsgEchoData(TypedDict):
    message_id: int


class SendMsgEcho(Echo):

    class Model(Echo.Model):
        data: _SendMsgEchoData | None

    data: _SendMsgEchoData | None


class _SendForwardMsgEchoData(TypedDict):
    message_id: int
    forward_id: str


class SendForwardMsgEcho(Echo):

    class Model(Echo.Model):
        data: _SendForwardMsgEchoData | None

    data: _SendForwardMsgEchoData | None


class _GetMsgEchoData(TypedDict):
    time: int
    message_type: Literal["private", "group"]
    message_id: int
    real_id: int


class _GetMsgEchoDataInterface(_GetMsgEchoData):
    sender: _MessageSender | _GroupMessageSender
    message: list[Segment]


class GetMsgEcho(Echo):

    class Model(Echo.Model):
        data: _GetMsgEchoData | None

    data: _GetMsgEchoDataInterface | None

    def __init__(self, **kv_pairs: Any) -> None:
        super().__init__(**kv_pairs)
        if self.data is None:
            return

        data = kv_pairs["data"]
        msgs = data["message"]
        segs: list[Segment]
        if isinstance(msgs, str):
            segs = Segment.__resolve_cq__(msgs)
        else:
            segs = [Segment.resolve(seg_dic["type"], seg_dic["data"]) for seg_dic in msgs]

        sender: _MessageSender | _GroupMessageSender
        if data["message_type"] == "private":
            sender = _MessageSender(**data["sender"])
        else:
            sender = _GroupMessageSender(**data["sender"])

        self.data["message"] = segs
        self.data["sender"] = sender


class _GetForwardMsgEchoData(TypedDict): ...


class _GetForwardMsgEchoDataInterface(_GetForwardMsgEchoData):
    message: list[NodeSegment]


class GetForwardMsgEcho(Echo):

    class Model(Echo.Model):
        data: _GetForwardMsgEchoData | None

    data: _GetForwardMsgEchoDataInterface | None

    def __init__(self, **kv_pairs: Any) -> None:
        super().__init__(**kv_pairs)
        if self.data is None:
            return

        data = kv_pairs["data"]
        msgs = data["message"]
        segs: list[Segment]
        if isinstance(msgs, str):
            segs = Segment.__resolve_cq__(msgs)
        else:
            segs = [Segment.resolve(seg_dic["type"], seg_dic["data"]) for seg_dic in msgs]

        self.data["message"] = cast(list[NodeSegment], segs)


class _GetLoginInfoEchoData(TypedDict):
    user_id: int
    nickname: str


class GetLoginInfoEcho(Echo):

    class Model(Echo.Model):
        data: _GetLoginInfoEchoData | None

    data: _GetLoginInfoEchoData | None


class _GetStrangerInfoEchoData(TypedDict):
    user_id: int
    nickname: str
    sex: Literal["male", "female", "unknown"]
    age: int


class GetStrangerInfoEcho(Echo):

    class Model(Echo.Model):
        data: _GetStrangerInfoEchoData | None

    data: _GetStrangerInfoEchoData | None


class _GetFriendListEchoElem(TypedDict):
    user_id: int
    nickname: str
    remark: str


class GetFriendListEcho(Echo):

    class Model(Echo.Model):
        data: list[_GetFriendListEchoElem] | None

    data: list[_GetFriendListEchoElem] | None


class _GetGroupInfoEchoData(TypedDict):
    group_id: int
    group_name: str
    member_count: int
    max_member_count: int


class GetGroupInfoEcho(Echo):

    class Model(Echo.Model):
        data: _GetGroupInfoEchoData | None

    data: _GetGroupInfoEchoData | None


class GetGroupListEcho(Echo):

    class Model(Echo.Model):
        data: list[_GetGroupInfoEchoData] | None

    data: list[_GetGroupInfoEchoData] | None


class _GetGroupMemberInfoEchoData(TypedDict):
    group_id: int
    user_id: int
    nickname: str
    card: str
    sex: str
    age: int
    area: str
    join_time: int
    last_sent_time: int
    level: str
    role: Literal["owner", "admin", "member"]
    unfriendly: bool
    title: str
    title_expire_time: int
    card_changeable: bool


class GetGroupMemberInfoEcho(Echo):

    class Model(Echo.Model):
        data: _GetGroupMemberInfoEchoData | None

    data: _GetGroupMemberInfoEchoData | None


class GetGroupMemberListEcho(Echo):

    class Model(Echo.Model):
        data: list[_GetGroupMemberInfoEchoData] | None

    data: list[_GetGroupMemberInfoEchoData] | None


class _CurrentTalkativeData(TypedDict):
    user_id: int
    nickname: str
    avatar: str
    day_count: int


class _OtherListData(TypedDict):
    user_id: int
    nickname: str
    avatar: str
    description: str


class _GetGroupHonorInfoEchoData(TypedDict):
    group_id: int
    current_talkative: _CurrentTalkativeData | None
    talkative_list: list[_OtherListData] | None
    performer_list: list[_OtherListData] | None
    legend_list: list[_OtherListData] | None
    strong_newbie_list: list[_OtherListData] | None
    emotion_list: list[_OtherListData] | None


class GetGroupHonorInfoEcho(Echo):

    class Model(Echo.Model):
        data: _GetGroupHonorInfoEchoData | None

    data: _GetGroupHonorInfoEchoData | None


class _GetCookiesEchoData(TypedDict):
    cookies: str


class GetCookiesEcho(Echo):

    class Model(Echo.Model):
        data: _GetCookiesEchoData | None

    data: _GetCookiesEchoData | None


class _GetCsrfTokenEchoData(TypedDict):
    token: int


class GetCsrfTokenEcho(Echo):

    class Model(Echo.Model):
        data: _GetCsrfTokenEchoData | None

    data: _GetCsrfTokenEchoData | None


class _GetCredentialsEchoData(TypedDict):
    cookies: str
    csrf_token: int


class GetCredentialsEcho(Echo):

    class Model(Echo.Model):
        data: _GetCredentialsEchoData | None

    data: _GetCredentialsEchoData | None


class _GetRecordEchoData(TypedDict):
    file: str


class GetRecordEcho(Echo):

    class Model(Echo.Model):
        data: _GetRecordEchoData | None

    data: _GetRecordEchoData | None


class _GetImageEchoData(TypedDict):
    file: str


class GetImageEcho(Echo):

    class Model(Echo.Model):
        data: _GetImageEchoData | None

    data: _GetImageEchoData | None


class _CanSendImageEchoData(TypedDict):
    yes: bool


class CanSendImageEcho(Echo):

    class Model(Echo.Model):
        data: _CanSendImageEchoData | None

    data: _CanSendImageEchoData | None


class _CanSendRecordEchoData(TypedDict):
    yes: bool


class CanSendRecordEcho(Echo):

    class Model(Echo.Model):
        data: _CanSendRecordEchoData | None

    data: _CanSendRecordEchoData | None


class _GetStatusEchoData(TypedDict):
    online: bool
    good: bool


class GetStatusEcho(Echo):

    class Model(Echo.Model):
        data: _GetStatusEchoData | None

    def __init__(self, **kv_pairs: Any) -> None:
        self._model: GetStatusEcho.Model
        super().__init__(**kv_pairs)
        if kv_pairs["data"] is None:
            return

        for k, v in kv_pairs["data"].items():
            if k not in cast(_GetStatusEchoData, self._model.data):
                cast(_GetStatusEchoData, self.data)[k] = v  # type: ignore[literal-required]

    data: _GetStatusEchoData | None


class _GetVersionInfoEchoData(TypedDict):
    app_name: str
    app_version: str
    protocol_version: str


class GetVersionInfoEcho(Echo):

    class Model(Echo.Model):
        data: _GetVersionInfoEchoData | None

    def __init__(self, **kv_pairs: Any) -> None:
        self._model: GetVersionInfoEcho.Model
        super().__init__(**kv_pairs)
        if kv_pairs["data"] is None:
            return

        for k, v in kv_pairs["data"].items():
            if k not in cast(_GetVersionInfoEchoData, self._model.data):
                cast(_GetVersionInfoEchoData, self.data)[k] = v  # type: ignore[literal-required]

    data: _GetVersionInfoEchoData | None



from tests.base import *


@fixture
def int_val() -> int:
    return 1


@aiofixture
async def test_v() -> int:
    return 5


async def test_loop(current_loop: aio.AbstractEventLoop, int_val: int) -> None:
    assert int_val == 1
    assert current_loop is aio.get_running_loop()


async def test_loop2(current_loop: aio.AbstractEventLoop, test_v: int) -> None:
    assert test_v == 5
    assert current_loop is aio.get_running_loop()


async def test3(current_loop: aio.AbstractEventLoop) -> None:
    assert isinstance(current_loop, aio.BaseEventLoop)



from ._enum import LogicMode, VoidType
from .base import AsyncCallable, P, SyncOrAsyncCallable, T, T_co, is_type
from .cls import (
    BetterABC,
    BetterABCMeta,
    SingletonBetterABCMeta,
    SingletonMeta,
    abstractattr,
)



from __future__ import annotations

import base64
import json
import re
import warnings
from collections.abc import Mapping
from itertools import chain, zip_longest

from beartype.door import is_subhint
from pydantic import BaseModel, Discriminator, Tag, create_model
from typing_extensions import (
    Annotated,
    Any,
    Generic,
    Literal,
    Match,
    NotRequired,
    Self,
    TypedDict,
    TypeVar,
    cast,
    final,
    get_args,
    overload,
)

from melobot.adapter import content as mbcontent

SegTypeT = TypeVar("SegTypeT", bound=str, default=Any)
SegDataT = TypeVar("SegDataT", bound=Mapping[str, Any], default=Any)


def cq_filter_text(s: str) -> str:
    """cq 文本过滤函数

    可从 cq 字符串中过滤出纯文本消息部分

    :param s: cq 字符串
    :return: 纯文本消息部分
    """
    regex = re.compile(r"\[CQ:.*?\]")
    return regex.sub("", s)


def cq_escape(text: str) -> str:
    """cq 字符串特殊符号转义

    如：将 "&" 转义为 "&amp;"

    :param text: 需要转义的 cq 字符串
    :return: 转义特殊符号后的 cq 字符串
    """
    return (
        text.replace("&", "&amp;")
        .replace("[", "&#91;")
        .replace("]", "&#93;")
        .replace(",", "&#44;")
    )


def cq_anti_escape(text: str) -> str:
    """cq 字符串特殊符号逆转义

    如：将 "&amp;" 逆转义为 "&"

    :param text: 需要逆转义的 cq 字符串
    :return: 逆转义特殊符号后的 cq 字符串
    """
    return (
        text.replace("&#44;", ",")
        .replace("&#93;", "]")
        .replace("&#91;", "[")
        .replace("&amp;", "&")
    )


def _cq_to_dicts(s: str) -> list[dict[str, Any]]:
    cq_texts: list[str] = []

    def replace_func(m: Match) -> str:
        s, e = m.regs[0]
        cq_texts.append(m.string[s:e])
        return "\u0000"

    cq_regex = re.compile(r"\[CQ:.*?\]")

    no_cq_str = cq_regex.sub(replace_func, s)
    pure_texts = map(
        lambda x: f"[CQ:text,text={x}]" if x != "" else x,
        no_cq_str.split("\u0000"),
    )
    cq_entity_str: str = "".join(
        chain.from_iterable(zip_longest(pure_texts, cq_texts, fillvalue=""))
    )

    cq_entity: list[str] = cq_entity_str.split("]")[:-1]
    dicts: list[dict[str, Any]] = []

    for e in cq_entity:
        cq_parts = e.split(",")
        cq_type = cq_parts[0][4:]
        cq_data: dict[str, Any] = {}

        for param_pair in cq_parts[1:]:
            name, val = param_pair.split("=", maxsplit=1)
            if len(cq_entity) == 1 and cq_type == "text":
                cq_data[name] = val
            else:
                cq_data[name] = cq_anti_escape(val)

        if (
            cq_type == "node"
            and "content" in cq_data
            and isinstance(cq_data["content"], str)
        ):
            cq_data["content"] = [
                Segment.resolve(seg_dict["type"], seg_dict["data"])
                for seg_dict in _cq_to_dicts(cq_data["content"])
            ]

        dicts.append({"type": cq_type, "data": cq_data})

    return dicts


def _segment_to_cq(type: str, data: dict[str, Any]) -> str:
    if type == "text":
        return cast(str, data["text"])

    if type != "node":
        params = ",".join(f"{k}={cq_escape(str(v))}" for k, v in data.items())
    else:
        param_list: list[str] = []
        for k, v in data.items():
            if k == "content":
                inner_cq = "".join(
                    _segment_to_cq(inner_seg.type, inner_seg.data) for inner_seg in v
                )
                param_list.append(f"{k}={cq_escape(inner_cq)}")
            else:
                param_list.append(f"{k}={cq_escape(str(v))}")
        params = ",".join(param_list)

    s = f"[CQ:{type}"
    if params != "":
        s += f",{params}"
    s += "]"
    return s


def base64_encode(data: bytes) -> str:
    code = "base64://"
    code += base64.b64encode(data).decode("utf-8")
    return code


def segs_to_contents(message: list[Segment]) -> list[mbcontent.Content]:
    contents: list[mbcontent.Content] = []
    for seg in message:
        if isinstance(seg, TextSegment):
            contents.append(mbcontent.TextContent(seg.data["text"]))

        elif isinstance(seg, ImageRecvSegment):
            contents.append(
                mbcontent.ImageContent(name=seg.data["file"], url=str(seg.data["url"]))
            )

        elif isinstance(seg, RecordRecvSegment):
            contents.append(
                mbcontent.VoiceContent(name=seg.data["file"], url=str(seg.data["url"]))
            )

        elif isinstance(seg, VideoRecvSegment):
            contents.append(
                mbcontent.VideoContent(name=seg.data["file"], url=str(seg.data["url"]))
            )

        elif isinstance(seg, AtSegment):
            contents.append(
                mbcontent.ReferContent(
                    prompt=str(seg.data["qq"]), flag=seg.data["qq"], contents=()
                )
            )

        elif isinstance(seg, ShareSegment):
            contents.append(
                mbcontent.ResourceContent(
                    name=seg.data["title"], url=str(seg.data["url"])
                )
            )

        else:
            continue

    return contents


def contents_to_segs(contents: list[mbcontent.Content]) -> list[Segment]:
    segments: list[Segment] = []
    for c in contents:
        if isinstance(c, mbcontent.TextContent):
            segments.append(TextSegment(c.text))

        elif isinstance(c, mbcontent.ImageContent):
            if c.val:
                file = base64_encode(c.val)
                segments.append(ImageSendSegment(file=file, cache=0))
            else:
                segments.append(ImageSendSegment(file=cast(str, c.url), cache=0))

        elif isinstance(c, mbcontent.VoiceContent):
            if c.val:
                file = base64_encode(c.val)
                segments.append(RecordSendSegment(file=file, cache=0))
            else:
                segments.append(RecordSendSegment(file=cast(str, c.url), cache=0))

        elif isinstance(c, mbcontent.AudioContent):
            if c.val:
                file = base64_encode(c.val)
                segments.append(RecordSendSegment(file=file, cache=0))
            else:
                segments.append(RecordSendSegment(file=cast(str, c.url), cache=0))

        elif isinstance(c, mbcontent.VideoContent):
            if c.val:
                file = base64_encode(c.val)
                segments.append(VideoSendSegment(file=file, cache=0))
            else:
                segments.append(VideoSendSegment(file=cast(str, c.url), cache=0))

        elif isinstance(c, mbcontent.MediaContent):
            if c.val:
                segments.append(TextSegment(f"[OneBot v11 media: {c.name}]"))
            else:
                segments.append(ShareSegment(url=cast(str, c.url), title=c.name))

        elif isinstance(c, mbcontent.FileContent):
            segments.append(TextSegment(repr(c)))

        elif isinstance(c, mbcontent.ReferContent):
            segments.append(TextSegment(repr(c)))

        elif isinstance(c, mbcontent.ResourceContent):
            segments.append(ShareSegment(url=c.url, title=c.name))

        else:
            continue

    return segments


TypeT = TypeVar("TypeT", bound=str, default=Any)
DataT = TypeVar("DataT", bound=Mapping[str, Any], default=Any)


class Segment(Generic[SegTypeT, SegDataT]):

    class Model(BaseModel):
        type: str
        data: dict

    def __init__(self, seg_type: SegTypeT, **seg_data: Any) -> None:
        self.raw = {"type": seg_type, "data": seg_data}
        self._model = self.Model(
            type=seg_type,
            data={k: v for k, v in seg_data.items() if v is not None},
        )

    @classmethod
    @final
    def add_type(
        cls, seg_type_hint: type[TypeT], seg_data_hint: type[DataT]
    ) -> type[_CustomSegInterface[TypeT, DataT]]:  # type: ignore[type-var]
        if cls is not Segment:
            raise ValueError(
                f"只能使用 {Segment.__name__} 类的 {Segment.add_type.__name__} 方法"
            )
        if not is_subhint(seg_type_hint, Literal):
            raise ValueError("新消息段的类型标注必须为 Literal")
        if not is_subhint(seg_data_hint, TypedDict):
            raise ValueError("新消息段的类型标注必须为 TypedDict")

        hint_args = get_args(seg_type_hint)
        if len(hint_args) != 1:
            raise ValueError("新消息段的类型标注必须只有一个字面量")

        type_name: str = hint_args[0]
        stand_name = type_name.lower().capitalize()
        type_classname = f"{stand_name}Segment"
        type_dataname = f"_{stand_name}Data"

        if type_classname in {subcls.__name__ for subcls in cls.__subclasses__()}:
            raise ValueError(f"类型为 {type_name} 的消息段类型已经存在")

        seg_cls = type(
            type_classname,
            (_CustomSegInterface,),
            {
                "Model": create_model(
                    type_dataname,
                    type=(seg_type_hint, ...),
                    data=(seg_data_hint, ...),
                ),
                "SegTypeVal": type_name,
            },
        )
        setattr(
            seg_cls,
            Segment.resolve.__name__,
            lambda _, seg_data: seg_cls(**seg_data),
        )
        return cast(type[_CustomSegInterface], seg_cls)

    @property
    def type(self) -> SegTypeT:
        return cast(SegTypeT, self._model.type)

    @property
    def data(self) -> SegDataT:
        return cast(SegDataT, self._model.data)

    @classmethod
    def resolve(cls, seg_type: Any, seg_data: Any) -> Segment:
        cls_name = f"{seg_type.lower().capitalize()}Segment"
        cls_map = {
            subcls.__name__: subcls
            for subcls in cls.__subclasses__() + _CustomSegInterface.__subclasses__()
        }
        if cls_name in cls_map:
            return cls_map[cls_name].resolve(seg_type, seg_data)
        return cls(seg_type, **seg_data)

    @classmethod
    def __resolve_cq__(cls, cq_str: str) -> list[Segment]:
        dicts = _cq_to_dicts(cq_str)
        segs = [cls.resolve(dic["type"], dic["data"]) for dic in dicts]
        return segs

    def to_cq(self) -> str:
        return _segment_to_cq(self.type, cast(dict[str, Any], self.data))

    def to_dict(self, force_str: bool = False) -> dict[str, Any]:
        dic: dict[str, Any] = self._model.model_dump()
        if not force_str:
            return dic

        for k, v in dic["data"].items():
            dic["data"][k] = str(v)
        return dic

    def to_json(self, force_str: bool = False) -> str:
        return json.dumps(self.to_dict(force_str), ensure_ascii=False)


class _CustomSegInterface(Segment[TypeT, DataT]):
    SegTypeVal: TypeT

    def __init__(self, **seg_data: DataT) -> None:
        super().__init__(self.__class__.SegTypeVal, **seg_data)


class _TextData(TypedDict):
    text: str


class TextSegment(Segment[Literal["text"], _TextData]):

    class Model(BaseModel):
        type: Literal["text"]
        data: _TextData

    def __init__(self, text: str, **kwargs: Any) -> None:
        super().__init__("text", text=text, **kwargs)

    @classmethod
    def resolve(cls, seg_type: Literal["text"], seg_data: _TextData) -> Self:
        return cls(**seg_data)

    def to_cq(self, escape: bool = False) -> str:
        text = super().to_cq()
        if escape:
            text = cq_escape(text)
        return text


class _FaceData(TypedDict):
    id: int


class FaceSegment(Segment[Literal["face"], _FaceData]):

    class Model(BaseModel):
        type: Literal["face"]
        data: _FaceData

    def __init__(self, id: int, **kwargs: Any) -> None:
        super().__init__("face", id=id, **kwargs)

    @classmethod
    def resolve(cls, seg_type: Literal["face"], seg_data: _FaceData) -> Self:
        return cls(**seg_data)


class _ImageSendData(TypedDict):
    file: str
    type: NotRequired[Literal["flash"]]
    cache: NotRequired[Literal[0, 1]]
    proxy: NotRequired[Literal[0, 1]]
    timeout: NotRequired[int]


class _ImageRecvData(TypedDict):
    file: str
    type: NotRequired[Literal["flash"]]
    url: str


class ImageSegment(Segment[Literal["image"], _ImageSendData | _ImageRecvData]):

    class Model(BaseModel):
        type: Literal["image"]
        data: _ImageSendData | _ImageRecvData

    @overload
    def __init__(
        self,
        *,
        file: str,
        type: Literal["flash"] | None = None,
        cache: Literal[0, 1] | None = None,
        proxy: Literal[0, 1] | None = None,
        timeout: int | None = None,
    ) -> None: ...

    @overload
    def __init__(
        self, *, file: str, url: str, type: Literal["flash"] | None = None
    ) -> None: ...

    def __init__(self, **kv_pairs: Any) -> None:
        super().__init__("image", **kv_pairs)

    @classmethod
    def resolve(
        cls,
        seg_type: Literal["image"],
        seg_data: _ImageSendData | _ImageRecvData,
    ) -> ImageSegment:
        if "url" in seg_data:
            return ImageRecvSegment(**seg_data)
        return ImageSendSegment(**seg_data)


class ImageSendSegment(ImageSegment):
    def __init__(
        self,
        *,
        file: str,
        type: Literal["flash"] | None = None,
        cache: Literal[0, 1] | None = None,
        proxy: Literal[0, 1] | None = None,
        timeout: int | None = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(
            file=file, type=type, cache=cache, proxy=proxy, timeout=timeout, **kwargs
        )

    data: _ImageSendData


class ImageRecvSegment(ImageSegment):
    def __init__(
        self,
        *,
        file: str,
        url: str,
        type: Literal["flash"] | None = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(file=file, url=url, type=type, **kwargs)

    data: _ImageRecvData


class _RecordSendData(TypedDict):
    file: str
    magic: NotRequired[Literal[0, 1]]
    cache: NotRequired[Literal[0, 1]]
    proxy: NotRequired[Literal[0, 1]]
    timeout: NotRequired[int]


class _RecordRecvData(TypedDict):
    file: str
    magic: NotRequired[Literal[0, 1]]
    url: str


class RecordSegment(Segment[Literal["record"], _RecordSendData | _RecordRecvData]):

    class Model(BaseModel):
        type: Literal["record"]
        data: _RecordSendData | _RecordRecvData

    @overload
    def __init__(
        self,
        *,
        file: str,
        magic: Literal[0, 1] | None = None,
        cache: Literal[0, 1] | None = None,
        proxy: Literal[0, 1] | None = None,
        timeout: int | None = None,
    ) -> None: ...

    @overload
    def __init__(
        self, *, file: str, url: str, magic: Literal[0, 1] | None = None
    ) -> None: ...

    def __init__(self, **kv_pairs: Any) -> None:
        super().__init__("record", **kv_pairs)

    @classmethod
    def resolve(
        cls, seg_type: Literal["record"], seg_data: _RecordSendData | _RecordRecvData
    ) -> RecordSegment:
        if "url" in seg_data:
            return RecordRecvSegment(**seg_data)
        return RecordSendSegment(**seg_data)


class RecordSendSegment(RecordSegment):
    def __init__(
        self,
        *,
        file: str | str,
        magic: Literal[0, 1] | None = None,
        cache: Literal[0, 1] | None = None,
        proxy: Literal[0, 1] | None = None,
        timeout: int | None = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(
            file=file, magic=magic, cache=cache, proxy=proxy, timeout=timeout, **kwargs
        )

    data: _RecordSendData


class RecordRecvSegment(RecordSegment):
    def __init__(
        self,
        *,
        file: str,
        url: str,
        magic: Literal[0, 1] | None = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(file=file, url=url, magic=magic, **kwargs)

    data: _RecordRecvData


class _VideoSendData(TypedDict):
    file: str
    cache: NotRequired[Literal[0, 1]]
    proxy: NotRequired[Literal[0, 1]]
    timeout: NotRequired[int]


class _VideoRecvData(TypedDict):
    file: str
    url: str


class VideoSegment(Segment[Literal["video"], _VideoSendData | _VideoRecvData]):

    class Model(BaseModel):
        type: Literal["video"]
        data: _VideoSendData | _VideoRecvData

    @overload
    def __init__(
        self,
        *,
        file: str,
        cache: Literal[0, 1] | None = None,
        proxy: Literal[0, 1] | None = None,
        timeout: int | None = None,
    ) -> None: ...

    @overload
    def __init__(self, *, file: str, url: str) -> None: ...

    def __init__(self, **kv_pairs: Any) -> None:
        super().__init__("video", **kv_pairs)

    @classmethod
    def resolve(
        cls, seg_type: Literal["video"], seg_data: _VideoSendData | _VideoRecvData
    ) -> VideoSegment:
        if "url" in seg_data:
            return VideoRecvSegment(**seg_data)
        return VideoSendSegment(**seg_data)


class VideoSendSegment(VideoSegment):
    def __init__(
        self,
        *,
        file: str,
        cache: Literal[0, 1] | None = None,
        proxy: Literal[0, 1] | None = None,
        timeout: int | None = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(file=file, cache=cache, proxy=proxy, timeout=timeout, **kwargs)

    data: _VideoSendData


class VideoRecvSegment(VideoSegment):
    def __init__(self, *, file: str, url: str, **kwargs: Any) -> None:
        super().__init__(file=file, url=url, **kwargs)

    data: _VideoRecvData


class _AtData(TypedDict):
    qq: int | Literal["all"]


class AtSegment(Segment[Literal["at"], _AtData]):

    class Model(BaseModel):
        type: Literal["at"]
        data: _AtData

    def __init__(self, qq: int | Literal["all"], **kwargs: Any) -> None:
        super().__init__("at", qq=qq, **kwargs)

    @classmethod
    def resolve(cls, seg_type: Literal["at"], seg_data: _AtData) -> AtSegment:
        return cls(**seg_data)


class _RpsData(TypedDict): ...


class RpsSegment(Segment[Literal["rps"], _RpsData]):

    class Model(BaseModel):
        type: Literal["rps"]
        data: _RpsData

    def __init__(self, **kwargs: Any) -> None:
        super().__init__("rps", **kwargs)

    @classmethod
    def resolve(cls, seg_type: Literal["rps"], seg_data: _RpsData) -> RpsSegment:
        return cls()


class _DictData(TypedDict): ...


class DiceSegment(Segment[Literal["dice"], _DictData]):

    class Model(BaseModel):
        type: Literal["dice"]
        data: _DictData

    def __init__(self, **kwargs: Any) -> None:
        super().__init__("dice", **kwargs)

    @classmethod
    def resolve(cls, seg_type: Literal["dice"], seg_data: _DictData) -> DiceSegment:
        return cls()


class _ShakeData(TypedDict): ...


class ShakeSegment(Segment[Literal["shake"], _ShakeData]):

    class Model(BaseModel):
        type: Literal["shake"]
        data: _ShakeData

    def __init__(self, **kwargs: Any) -> None:
        super().__init__("shake", **kwargs)

    @classmethod
    def resolve(cls, seg_type: Literal["shake"], seg_data: _ShakeData) -> ShakeSegment:
        return cls()


class _PokeSendData(TypedDict):
    type: str
    id: int


class _PokeRecvData(TypedDict):
    type: str
    id: int
    name: int


class PokeSegment(Segment[Literal["poke"], _PokeSendData | _PokeRecvData]):

    class Model(BaseModel):
        type: Literal["poke"]
        data: _PokeSendData | _PokeRecvData

    @overload
    def __init__(self, *, type: str, id: int) -> None: ...

    @overload
    def __init__(self, *, type: str, id: int, name: int) -> None: ...

    def __init__(self, **kv_pairs: Any) -> None:
        super().__init__("poke", **kv_pairs)

    @classmethod
    def resolve(
        cls, seg_type: Literal["poke"], seg_data: _PokeSendData | _PokeRecvData
    ) -> PokeSegment:
        if "name" in seg_data:
            return PokeRecvSegment(**seg_data)  # type: ignore[call-arg]
        return PokeSendSegment(**seg_data)


class PokeSendSegment(PokeSegment):
    def __init__(self, type: str, id: int, **kwargs: Any) -> None:
        super().__init__(type=type, id=id, **kwargs)

    data: _PokeSendData


class PokeRecvSegment(PokeSegment):
    def __init__(self, type: str, id: int, name: int, **kwargs: Any) -> None:
        super().__init__(type=type, id=id, name=name, **kwargs)

    data: _PokeRecvData


class _AnonymousData(TypedDict):
    ignore: NotRequired[Literal[0, 1]]


class AnonymousSegment(Segment[Literal["anonymous"], _AnonymousData]):

    class Model(BaseModel):
        type: Literal["anonymous"]
        data: _AnonymousData

    def __init__(self, ignore: Literal[0, 1] | None = None, **kwargs: Any) -> None:
        super().__init__("anonymous", ignore=ignore, **kwargs)

    @classmethod
    def resolve(
        cls, seg_type: Literal["anonymous"], seg_data: _AnonymousData
    ) -> AnonymousSegment:
        return cls(**seg_data)


class _ShareData(TypedDict):
    url: str
    title: str
    content: NotRequired[str]
    image: NotRequired[str]


class ShareSegment(Segment[Literal["share"], _ShareData]):

    class Model(BaseModel):
        type: Literal["share"]
        data: _ShareData

    def __init__(
        self,
        url: str,
        title: str,
        content: str | None = None,
        image: str | None = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(
            "share", url=url, title=title, content=content, image=image, **kwargs
        )

    @classmethod
    def resolve(cls, seg_type: Literal["share"], seg_data: _ShareData) -> ShareSegment:
        return cls(**seg_data)


class _ContactFriendData(TypedDict):
    type: Literal["qq"]
    id: int


class _ContactGroupData(TypedDict):
    type: Literal["group"]
    id: int


class ContactSegment(Segment[Literal["contact"], _ContactFriendData | _ContactGroupData]):

    class Model(BaseModel):
        type: Literal["contact"]
        data: _ContactFriendData | _ContactGroupData

    def __init__(self, type: Literal["qq", "group"], id: int, **kwargs: Any) -> None:
        super().__init__("contact", type=type, id=id, **kwargs)

    @classmethod
    def resolve(
        cls,
        seg_type: Literal["contact"],
        seg_data: _ContactFriendData | _ContactGroupData,
    ) -> ContactSegment:
        if seg_data["type"] == "qq":
            return ContactFriendSegment(**seg_data)  # type:ignore[misc]
        return ContactGroupSegment(**seg_data)  # type:ignore[misc]


class ContactFriendSegment(ContactSegment):
    def __init__(self, id: int, **kwargs: Any) -> None:
        super().__init__(type="qq", id=id, **kwargs)

    data: _ContactFriendData


class ContactGroupSegment(ContactSegment):
    def __init__(self, id: int, **kwargs: Any) -> None:
        super().__init__(type="group", id=id, **kwargs)

    data: _ContactGroupData


class _LocationData(TypedDict):
    lat: float
    lon: float
    title: NotRequired[str]
    content: NotRequired[str]


class LocationSegment(Segment[Literal["location"], _LocationData]):

    class Model(BaseModel):
        type: Literal["location"]
        data: _LocationData

    def __init__(
        self,
        lat: float,
        lon: float,
        title: str | None = None,
        content: str | None = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(
            "location", lat=lat, lon=lon, title=title, content=content, **kwargs
        )

    @classmethod
    def resolve(
        cls, seg_type: Literal["location"], seg_data: _LocationData
    ) -> LocationSegment:
        return cls(**seg_data)


class _MusicData(TypedDict):
    type: Literal["qq", "163", "xm"]
    id: str


class _MusicCustomData(TypedDict):
    type: Literal["custom"]
    url: str
    audio: str
    title: str
    content: NotRequired[str]
    image: NotRequired[str]


class MusicSegment(Segment[Literal["music"], _MusicData | _MusicCustomData]):

    class Model(BaseModel):
        type: Literal["music"]
        data: _MusicData | _MusicCustomData

    @overload
    def __init__(self, *, type: Literal["qq", "163", "xm"], id: str) -> None: ...

    @overload
    def __init__(
        self,
        *,
        type: Literal["custom"],
        url: str,
        audio: str,
        title: str,
        content: str | None = None,
        image: str | None = None,
    ) -> None: ...

    def __init__(self, **kv_pairs: Any) -> None:
        super().__init__("music", **kv_pairs)

    @classmethod
    def resolve(
        cls, seg_type: Literal["music"], seg_data: _MusicData | _MusicCustomData
    ) -> MusicSegment:
        if seg_data["type"] == "custom":
            return MusicCustomSegment(**seg_data)
        return MusicPlatformSegment(**seg_data)


class MusicPlatformSegment(MusicSegment):
    def __init__(
        self, *, type: Literal["qq", "163", "xm"], id: str, **kwargs: Any
    ) -> None:
        super().__init__(type=type, id=id, **kwargs)

    data: _MusicData


class MusicCustomSegment(MusicSegment):
    def __init__(
        self,
        *,
        type: Literal["custom"],
        url: str,
        audio: str,
        title: str,
        content: str | None = None,
        image: str | None = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(
            type=type,
            url=url,
            audio=audio,
            title=title,
            content=content,
            image=image,
            **kwargs,
        )

    data: _MusicCustomData


class _ReplyData(TypedDict):
    id: str


class ReplySegment(Segment[Literal["reply"], _ReplyData]):

    class Model(BaseModel):
        type: Literal["reply"]
        data: _ReplyData

    def __init__(self, id: str, **kwargs: Any) -> None:
        super().__init__("reply", id=id, **kwargs)

    @classmethod
    def resolve(cls, seg_type: Literal["reply"], seg_data: _ReplyData) -> ReplySegment:
        return cls(**seg_data)


class _ForwardData(TypedDict):
    id: str


class ForwardSegment(Segment[Literal["forward"], _ForwardData]):

    class Model(BaseModel):
        type: Literal["forward"]
        data: _ForwardData

    def __init__(self, id: str, **kwargs: Any) -> None:
        super().__init__("forward", id=id, **kwargs)

    @classmethod
    def resolve(
        cls, seg_type: Literal["forward"], seg_data: _ForwardData
    ) -> ForwardSegment:
        return cls(**seg_data)


class _NodeReferData(TypedDict):
    id: str


class _NodeStdCustomData(TypedDict):
    user_id: int
    nickname: str


class _NodeStdCustomDataInterface(_NodeStdCustomData):
    content: list[Segment]


class _NodeGocqCustomData(TypedDict):
    uin: int
    name: str


class _NodeGocqCustomDataInterface(_NodeGocqCustomData):
    content: list[Segment]


class NodeSegment(
    Segment[
        Literal["node"],
        _NodeReferData | _NodeStdCustomDataInterface | _NodeGocqCustomDataInterface,
    ]
):

    class Model(BaseModel):
        type: Literal["node"]
        data: Annotated[
            Annotated[_NodeReferData, Tag("0")]
            | Annotated[_NodeStdCustomData, Tag("1")]
            | Annotated[_NodeGocqCustomData, Tag("2")],
            Discriminator(
                lambda data: "0" if "id" in data else "1" if "user_id" in data else "2"
            ),
        ]

    @overload
    def __init__(self, *, id: str) -> None: ...

    @overload
    def __init__(
        self,
        *,
        uin: int,
        name: str,
        content: list[Segment] | list[dict],
        use_std: bool = False,
    ) -> None: ...

    def __init__(self, **kv_pairs: Any) -> None:
        std: bool = kv_pairs.pop("use_std")
        id: str | None = kv_pairs.pop("id", None)
        content = kv_pairs.pop("content")
        _content: list[Segment]
        if len(content) and isinstance(content[0], dict):
            _content = [
                Segment.resolve(seg_dic["type"], seg_dic["data"]) for seg_dic in content
            ]
        else:
            _content = content

        if id:
            super().__init__("node", id=id)
            return

        if not std:
            super().__init__("node", uin=kv_pairs["uin"], name=kv_pairs["name"])
        else:
            super().__init__("node", user_id=kv_pairs["uin"], nickname=kv_pairs["name"])
        self.data["content"] = _content  # type: ignore[typeddict-unknown-key]

    @classmethod
    def resolve(
        cls,
        seg_type: Literal["node"],
        seg_data: (
            _NodeReferData | _NodeStdCustomDataInterface | _NodeGocqCustomDataInterface
        ),
    ) -> NodeSegment:
        if "id" in seg_data:
            return NodeReferSegment(**seg_data)
        if "user_id" in seg_data:
            return NodeStdCustomSegment(**seg_data)  # type: ignore[arg-type]
        return NodeGocqCustomSegment(**seg_data)  # type: ignore[arg-type]

    def to_dict(self, force_str: bool = False) -> dict[str, Any]:
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", UserWarning)
            dic = super().to_dict(force_str=False)

        if "content" in self.data:
            dic["data"]["content"] = [
                s.to_dict(force_str) for s in self.data["content"]  # type: ignore[typeddict-item]
            ]

        if not force_str:
            return dic

        for k, v in dic["data"].items():
            if k != "content":
                dic["data"][k] = str(v)

        return dic


class NodeReferSegment(NodeSegment):
    def __init__(self, id: str, **kwargs: Any) -> None:
        super().__init__(id=id, **kwargs)

    data: _NodeReferData


class NodeStdCustomSegment(NodeSegment):
    def __init__(
        self, user_id: int, nickname: str, content: list[Segment], **kwargs: Any
    ) -> None:
        super().__init__(
            uin=user_id, name=nickname, content=content, use_std=True, **kwargs
        )

    data: _NodeStdCustomDataInterface


class NodeGocqCustomSegment(NodeSegment):
    def __init__(
        self, uin: int, name: str, content: list[Segment], **kwargs: Any
    ) -> None:
        super().__init__(uin=uin, name=name, content=content, use_std=False, **kwargs)

    data: _NodeGocqCustomDataInterface


class _XmlData(TypedDict):
    data: str


class XmlSegment(Segment[Literal["xml"], _XmlData]):

    class Model(BaseModel):
        type: Literal["xml"]
        data: _XmlData

    def __init__(self, data: str, **kwargs: Any) -> None:
        super().__init__("xml", data=data, **kwargs)

    @classmethod
    def resolve(cls, seg_type: Literal["xml"], seg_data: _XmlData) -> XmlSegment:
        return cls(**seg_data)


class _JsonData(TypedDict):
    data: str


class JsonSegment(Segment[Literal["json"], _JsonData]):

    class Model(BaseModel):
        type: Literal["json"]
        data: _JsonData

    def __init__(self, data: str, **kwargs: Any) -> None:
        super().__init__("json", data=data, **kwargs)

    @classmethod
    def resolve(cls, seg_type: Literal["json"], seg_data: _JsonData) -> JsonSegment:
        return cls(**seg_data)



import re
import sys
import types

from typing_extensions import Any, Callable, Protocol, cast

from .base import GenericLogger, Logger, LogLevel


class LazyLogMethod(Protocol):
    def __call__(
        self,
        msg: str,
        *arg_getters: Callable[[], Any],
        level: LogLevel,
        with_exc: bool = False,
    ) -> None:
        """懒惰日志抽象方法

        继承并实现该类的 __call__ 方法，即可用于日志器修补

        :param msg: 日志消息
        :param level: 日志等级
        :param with_exc: 输出时是否附加异常信息
        """
        raise NotImplementedError


def logger_patch(logger: Any, lazy_meth: LazyLogMethod) -> GenericLogger:
    """对指定的日志器进行修补操作，使其可以被用于 melobot 内部的日志记录

    :param logger: 任意已经实现 `debug`, `info`, `warning`, `error`, `critical`, `exception` 接口的日志器
    :param lazy_meth: 修补方法
    """
    setattr(logger, Logger.generic_lazy.__name__, lazy_meth)
    setattr(
        logger, Logger.generic_obj.__name__, types.MethodType(Logger.generic_obj, logger)
    )
    return cast(GenericLogger, logger)


class StandardPatch(LazyLogMethod):
    """用于修补 logging.Logger 的日志修补"""

    def __init__(self, logger: Any) -> None:
        """初始化一个标准日志器修补

        :param logger: 标准日志器对象（`logging.Logger`）
        """
        super().__init__()
        self.logger = logger

    def __call__(
        self,
        msg: str,
        *arg_getters: Callable[[], Any],
        level: LogLevel,
        with_exc: bool = False,
    ) -> None:
        if not self.logger.isEnabledFor(level):
            return
        exc = sys.exc_info() if with_exc else None
        self.logger._log(level, msg, tuple(g() for g in arg_getters), exc_info=exc)


class LoguruPatch(LazyLogMethod):
    """用于修补 loguru 日志器的日志修补"""

    def __init__(self, logger: Any) -> None:
        """初始化一个 loguru 日志器修补

        :param logger: loguru 日志器对象
        """
        super().__init__()
        self.logger = logger
        self.pattern = re.compile(r"%(?:[-+# 0]*\d*(?:\.\d+)?[hlL]?[diouxXeEfFgGcrs%])")

    def __call__(
        self,
        msg: str,
        *arg_getters: Callable[[], Any],
        level: LogLevel,
        with_exc: bool = False,
    ) -> None:
        logger = self.logger.opt(lazy=True)
        msg = self.pattern.sub("{}", msg)

        if with_exc:
            logger.exception(msg, *arg_getters)
            return

        match level:
            case LogLevel.DEBUG:
                logger.debug(msg, *arg_getters)
            case LogLevel.INFO:
                logger.info(msg, *arg_getters)
            case LogLevel.WARNING:
                logger.warning(msg, *arg_getters)
            case LogLevel.ERROR:
                logger.error(msg, *arg_getters)
            case LogLevel.CRITICAL:
                logger.critical(msg, *arg_getters)
            case _:
                raise ValueError(f"无效的日志等级：{level}")


class StructlogPatch(LazyLogMethod):
    """用于修补 structlog 日志器的日志修补"""

    def __init__(self, logger: Any) -> None:
        """初始化一个 structlog 日志器修补

        :param logger: structlog 日志器对象
        """
        super().__init__()
        self.logger = logger

    def __call__(
        self,
        msg: str,
        *arg_getters: Callable[[], Any],
        level: LogLevel,
        with_exc: bool = False,
    ) -> None:
        args = tuple(g() for g in arg_getters)
        logger = self.logger
        if with_exc:
            logger.exception(msg, *args)
            return

        match level:
            case LogLevel.DEBUG:
                logger.debug(msg, *args)
            case LogLevel.INFO:
                logger.info(msg, *args)
            case LogLevel.WARNING:
                logger.warning(msg, *args)
            case LogLevel.ERROR:
                logger.error(msg, *args)
            case LogLevel.CRITICAL:
                logger.critical(msg, *args)
            case _:
                raise ValueError(f"无效的日志等级：{level}")



from asyncio import Future
from contextlib import contextmanager
from contextvars import ContextVar, Token
from dataclasses import dataclass, field
from enum import Enum

from typing_extensions import (
    TYPE_CHECKING,
    Any,
    Callable,
    Generator,
    Generic,
    Self,
    Union,
    cast,
)

from .exceptions import AdapterError, BotError, FlowError, LogError, SessionError
from .typ.base import T
from .typ.cls import SingletonMeta

if TYPE_CHECKING:
    from .adapter import Adapter, model
    from .bot.base import Bot
    from .handle.base import Flow, FlowNode
    from .io.base import AbstractInSource, OutSourceT
    from .log.base import GenericLogger
    from .session.base import Session, SessionStore
    from .session.option import Rule
    from .utils.parse import AbstractParseArgs


class Context(Generic[T], metaclass=SingletonMeta):
    """上下文对象，本质是对 :class:`contextvars.ContextVar` 操作的封装

    继承该基类，可以实现自己的上下文对象。
    任何时候不应该直接实例化该类，而是应该继承实现子类，再使用子类
    """

    def __init__(
        self,
        ctx_name: str,
        lookup_exc_cls: type[BaseException],
        lookup_exc_tip: str | None = None,
    ) -> None:
        """初始化一个上下文对象

        :param ctx_name: 上下文的名称（唯一）
        :param lookup_exc_cls: 当试图获取上下文值失败时，抛出的异常
        :param lookup_exc_tip: 当试图获取上下文值失败时，抛出异常的附加说明
        """
        if self.__class__ is Context:
            raise TypeError(
                f"任何时候都不应该直接实例化 {Context.__name__}，而应该实现子类"
            )

        self.__storage__ = ContextVar[T](ctx_name)
        self.lookup_exc_cls = lookup_exc_cls
        self.lookup_exc_tip = lookup_exc_tip

    def get(self) -> T:
        """在当前上下文中，获取本上下文对象的上下文值

        :return: 上下文对象的上下文值
        """
        try:
            return self.__storage__.get()
        except LookupError:
            raise self.lookup_exc_cls(self.lookup_exc_tip) from None

    def try_get(self) -> T | None:
        """与 :func:`get` 类似，但不存在上下文对象时返回 `None`

        :return: 上下文对象的上下文值
        """
        return self.__storage__.get(None)

    def add(self, ctx: T) -> Token[T]:
        """在当前上下文中，添加一个上下文值

        :param ctx: 上下文值
        :return: :class:`contextvars.Token` 对象
        """
        return self.__storage__.set(ctx)

    def remove(self, token: Token[T]) -> None:
        """移除当前上下文中的上下文值

        :param token: 添加时返回的 :class:`contextvars.Token` 对象
        """
        self.__storage__.reset(token)

    @contextmanager
    def unfold(self, obj: T) -> Generator[T, None, None]:
        """展开一个上下文值为 `obj` 的上下文环境，返回上下文管理器

        上下文管理器可 `yield` 上下文值，退出上下文管理器作用域后自动清理

        :param obj: 上下文值
        """
        token = self.add(obj)
        try:
            yield obj
        finally:
            self.remove(token)


_OutSrcFilterType = Callable[["OutSourceT"], bool]


class OutSrcFilterCtx(Context[_OutSrcFilterType]):
    def __init__(self) -> None:
        super().__init__("MELOBOT_OUT_SRC_FILTER", AdapterError)


class FlowRecordStage(Enum):
    """流记录阶段的枚举"""

    FLOW_START = "fs"
    FLOW_EARLY_FINISH = "fef"
    FLOW_FINISH = "ff"

    NODE_START = "ns"
    DEPENDS_NOT_MATCH = "dnm"
    BLOCK = "bl"
    STOP = "st"
    BYPASS = "by"
    REWIND = "re"
    NODE_EARLY_FINISH = "nef"
    NODE_FINISH = "nf"


@dataclass
class FlowRecord:
    """流记录"""

    stage: FlowRecordStage
    flow_name: str
    node_name: str
    event: "model.Event"
    prompt: str = ""


class FlowRecords(list[FlowRecord]):
    def append(self, snapshot: FlowRecord) -> None:
        super().append(snapshot)


class FlowStore(dict[str, Any]):
    """流存储，将会在流运行前初始化，运行结束后销毁"""


@dataclass
class FlowStatus:
    flow: "Flow"
    node: "FlowNode"
    next_valid: bool
    completion: "EventCompletion"
    records: FlowRecords = field(default_factory=FlowRecords)
    store: FlowStore = field(default_factory=FlowStore)


class FlowCtx(Context[FlowStatus]):
    def __init__(self) -> None:
        super().__init__(
            "MELOBOT_FLOW",
            FlowError,
            "此时不在活动的事件处理流中，无法获取处理流信息",
        )

    def get_event(self) -> "model.Event":
        session = SessionCtx().try_get()
        if session is not None:
            return session.event
        return self.get().completion.event

    def try_get_event(self) -> Union["model.Event", None]:
        session = SessionCtx().try_get()
        if session is not None:
            return session.event

        status = self.try_get()
        if status is not None:
            return status.completion.event
        return None

    def get_event_type(self) -> type["model.Event"]:
        from .adapter.model import Event

        return Event

    def get_records(self) -> tuple[FlowRecord, ...]:
        return tuple(self.get().records)

    def get_completion(self) -> "EventCompletion":
        return self.get().completion

    def get_store(self) -> FlowStore:
        return self.get().store

    def get_store_type(self) -> type[FlowStore]:
        return FlowStore


class BotCtx(Context["Bot"]):
    def __init__(self) -> None:
        super().__init__("MELOBOT_BOT", BotError, "此时未初始化 bot 实例，无法获取")

    def get_type(self) -> type["Bot"]:
        from .bot.base import Bot

        return Bot


class SessionCtx(Context["Session"]):
    def __init__(self) -> None:
        super().__init__(
            "MELOBOT_SESSION",
            SessionError,
            "此时不在活动的事件处理流中，无法获取会话信息",
        )

    def get_store(self) -> "SessionStore":
        return self.get().store

    def get_rule(self) -> "Rule":
        rule = self.get().rule
        assert rule is not None, "预期之外的会话规则为空"
        return rule

    def get_session_type(self) -> type["Session"]:
        from .session.base import Session

        return Session

    def get_store_type(self) -> type["SessionStore"]:
        from .session.base import SessionStore

        return SessionStore

    def get_rule_type(self) -> type["Rule"]:
        from .session.option import Rule

        return Rule


class LoggerCtx(Context["GenericLogger"]):
    def __init__(self) -> None:
        super().__init__("MELOBOT_LOGGER", LogError, "此时未初始化 logger 实例，无法获取")

    def get_type(self) -> type["GenericLogger"]:
        from .log.base import GenericLogger

        return GenericLogger


class EventOrigin:
    _FLAG_KEYS = (object(), object())

    def __init__(self, adapter: "Adapter", in_src: "AbstractInSource") -> None:
        self.adapter = adapter
        self.in_src = in_src

    @classmethod
    def set_origin(cls, event: "model.Event", origin: "EventOrigin") -> None:
        event.flag_set(cls._FLAG_KEYS[0], cls._FLAG_KEYS[1], origin)

    @classmethod
    def get_origin(cls, event: "model.Event") -> Self:
        origin = event.flag_get(cls._FLAG_KEYS[0], cls._FLAG_KEYS[1])
        return cast(Self, origin)


# 不使用 dataclass，不用重写任何方法就可哈希
class EventCompletion:
    def __init__(
        self,
        event: "model.Event",
        completed: Future[None],
        owner_flow: "Flow",
        under_session: bool = False,
    ) -> None:
        self.event = event
        self.completed = completed
        self.owner_flow = owner_flow
        self.under_session = under_session


class ActionManualSignalCtx(Context[bool]):
    def __init__(self) -> None:
        super().__init__("MELOBOT_ACTION_MANUAL_SIGNAL", AdapterError)


class ParseArgsCtx(Context["AbstractParseArgs"]):
    def __init__(self) -> None:
        super().__init__("MELOBOT_PARSE_ARGS", LookupError, "当前上下文中不存在解析参数")

    def get_args_type(self) -> type["AbstractParseArgs"]:
        from .utils.parse import AbstractParseArgs

        return AbstractParseArgs



from .base import (
    AbstractEchoFactory,
    AbstractEventFactory,
    AbstractOutputFactory,
    Adapter,
    AdapterLifeSpan,
)
from .content import Content
from .model import (
    Action,
    ActionChain,
    ActionHandle,
    ActionRetT,
    ActionT,
    Echo,
    EchoT,
    Event,
    EventT,
    TextEvent,
    open_chain,
)
